# modules/converter.py
# excel to word file
from .editing_functions import FUNCTION_MAP
import re
import random
from collections import defaultdict
from pathlib import Path
from tkinter import messagebox, Toplevel
import inspect
from docx import Document
from docx.shared import Inches, Pt, Cm, RGBColor
from docx.enum.text import WD_PARAGRAPH_ALIGNMENT, WD_LINE_SPACING, WD_BREAK
from docx.oxml import OxmlElement
from docx.oxml.ns import qn, nsdecls
from docx.enum.style import WD_STYLE_TYPE
from docx.enum.text import WD_ALIGN_PARAGRAPH
from docx.enum.table import WD_TABLE_ALIGNMENT
from docx.enum.table import WD_ALIGN_VERTICAL
from docx.enum.text import WD_COLOR_INDEX
from openpyxl import load_workbook
import shutil
import os.path
import tempfile
import unicodedata


# Import customtkinter components
from customtkinter import CTkLabel, CTkFrame, CTkButton




########################################################
################### 편집 함수  ###################  
########################################################


# Function to parse the 'question_part' data and extract table information
def parse_data(question_part):
    lines = question_part.strip().split('\n')[2:]  # Skip header and separator lines
    data = []
    for line in lines:
        columns = line.split('|')[1:-1]  # Split by '|' and remove first and last empty strings
        data.append([col.strip() for col in columns])  # Strip spaces and add to data list
    return data

# Function to insert data into a table in the document
def insert_data_into_table(document, data):
    max_cols = max(len(row) for row in data)
    table = document.add_table(rows=len(data) + 1, cols=max_cols)
    
    column_widths = [Inches(1), Inches(1), Inches(3), Inches(3)][:max_cols]  # Adjust based on the number of columns
    
    for row in table.rows:
        for idx, width in enumerate(column_widths):
            row.cells[idx].width = width

    # Add header row and format cells
    #for i, header in enumerate(["번호", "단어", "뜻", "예문"]):
    for i, header in enumerate(["영어", "뜻", "동의어", "반의어"]):
        cell = table.cell(0, i)
        cell.text = header
        set_cell_border(
            cell,
            top={"sz": 5, "val": "single", "color": "auto"},
            bottom={"sz": 5, "val": "single", "color": "auto"},
            left={"sz": 5, "val": "single", "color": "auto"},
            right={"sz": 5, "val": "single", "color": "auto"}
        )
    
    # Insert data rows and format cells
    for row_idx, row_data in enumerate(data, start=1):
        for col_idx, cell_data in enumerate(row_data):
            cell = table.cell(row_idx, col_idx)
            cell.text = cell_data
            set_cell_border(
                cell,
                top={"sz": 5, "val": "single", "color": "auto"},
                bottom={"sz": 5, "val": "single", "color": "auto"},
                left={"sz": 5, "val": "single", "color": "auto"},
                right={"sz": 5, "val": "single", "color": "auto"}
            )
    
    # Set table cell margins after formatting cells if needed
    set_table_cell_margins(table, top=3, bottom=3, left=5, right=5)



# Function to insert data into a table in the document
def insert_data_into_table_영영정의(document, data):
    max_cols = max(len(row) for row in data)
    table = document.add_table(rows=len(data) + 1, cols=max_cols)
    
    column_widths = [Inches(1), Inches(1), Inches(6)][:max_cols]  # Adjust based on the number of columns
    
    for row in table.rows:
        for idx, width in enumerate(column_widths):
            row.cells[idx].width = width

    # Add header row and format cells
    #for i, header in enumerate(["번호", "단어", "뜻", "예문"]):
    for i, header in enumerate(["영어", "뜻", "영영정의"]):
        cell = table.cell(0, i)
        cell.text = header
        set_cell_border(
            cell,
            top={"sz": 5, "val": "single", "color": "auto"},
            bottom={"sz": 5, "val": "single", "color": "auto"},
            left={"sz": 5, "val": "single", "color": "auto"},
            right={"sz": 5, "val": "single", "color": "auto"}
        )
    
    # Insert data rows and format cells
    for row_idx, row_data in enumerate(data, start=1):
        for col_idx, cell_data in enumerate(row_data):
            cell = table.cell(row_idx, col_idx)
            cell.text = cell_data
            set_cell_border(
                cell,
                top={"sz": 5, "val": "single", "color": "auto"},
                bottom={"sz": 5, "val": "single", "color": "auto"},
                left={"sz": 5, "val": "single", "color": "auto"},
                right={"sz": 5, "val": "single", "color": "auto"}
            )
    
    # Set table cell margins after formatting cells if needed
    set_table_cell_margins(table, top=3, bottom=3, left=5, right=5)


def set_cell_border(cell, **kwargs):
    tc = cell._tc
    tcPr = tc.get_or_add_tcPr()

    for edge in ('top', 'bottom', 'left', 'right', 'insideH', 'insideV'):
        edge_data = kwargs.get(edge)
        if edge_data:
            tag = 'w:{}'.format(edge)
            element = OxmlElement(tag)
            element.set(qn('w:val'), edge_data.get('val', 'single'))
            element.set(qn('w:sz'), str(edge_data.get('sz', 12)))
            element.set(qn('w:space'), str(edge_data.get('space', 0)))
            element.set(qn('w:color'), edge_data.get('color', 'auto'))
            tcPr.append(element)


def set_table_cell_margins(table, top=0, bottom=0, left=0, right=0):
    # Convert points to dxa (1 point = 20 dxa)
    top_dxa = str(int(top * 20))
    bottom_dxa = str(int(bottom * 20))
    left_dxa = str(int(left * 20))
    right_dxa = str(int(right * 20))

    for row in table.rows:
        for cell in row.cells:
            tcPr = cell._element.get_or_add_tcPr()
            tcMar = OxmlElement('w:tcMar')

            # Set top margin
            top_mar = OxmlElement('w:top')
            top_mar.set(qn('w:w'), top_dxa)
            top_mar.set(qn('w:type'), "dxa")
            tcMar.append(top_mar)

            # Set bottom margin
            bottom_mar = OxmlElement('w:bottom')
            bottom_mar.set(qn('w:w'), bottom_dxa)
            bottom_mar.set(qn('w:type'), "dxa")
            tcMar.append(bottom_mar)

            # Set left margin
            left_mar = OxmlElement('w:left')
            left_mar.set(qn('w:w'), left_dxa)
            left_mar.set(qn('w:type'), "dxa")
            tcMar.append(left_mar)

            # Set right margin
            right_mar = OxmlElement('w:right')
            right_mar.set(qn('w:w'), right_dxa)
            right_mar.set(qn('w:type'), "dxa")
            tcMar.append(right_mar)

            tcPr.append(tcMar)








def add_bold_brackets_to_cell_paragraph(cell, paragraph_texts):
    # Define the pattern to match brackets and numbers in front
    pattern = re.compile(r"\(\d+\)❮[^❯]*❯")

    # Iterate over each paragraph text
    for paragraph_text in paragraph_texts:
        # If it's the first paragraph, use the existing one; otherwise, create a new one
        if paragraph_texts.index(paragraph_text) == 0:
            para = cell.paragraphs[0]
            para.clear()  # Clear the existing paragraph to remove any default text
            para.style = 'Normal_modified'  # Apply the 'Normal_modified' style

        else:
            para = cell.add_paragraph(style='Normal_modified')

        last_idx = 0  # Start index of the last added run
        for match in pattern.finditer(paragraph_text):
            # Add the text before the match
            start, end = match.span()
            para.add_run(paragraph_text[last_idx:start])
            # Add the matched text in bold
            para.add_run(paragraph_text[start:end]).bold = True
            # Update the last index
            last_idx = end

        # Add any remaining text after the last match
        para.add_run(paragraph_text[last_idx:])


# [] 와 [[ ]] 둘 다 호환.
"""
def add_bold_underline_to_brackets(cell, paragraph_texts):
    # Define the pattern to match text within < >, [[ ]], or [ ]
    pattern = re.compile(r"\<([^>]*)\>|\[\[([^\]]*)\]\]|\[([^\]]*)\]")

    # Iterate over each paragraph text with enumeration for efficiency
    for idx, paragraph_text in enumerate(paragraph_texts):
        # If it's the first paragraph, use the existing one; otherwise, create a new one
        if idx == 0:
            para = cell.paragraphs[0]
            para.clear()  # Clear the existing paragraph to remove any default text
            para.style = 'Normal_modified'  # Apply the 'Normal_modified' style
        else:
            para = cell.add_paragraph(style='Normal_modified')

        last_idx = 0  # Start index of the last added run
        for match in pattern.finditer(paragraph_text):
            # Get the span of the matched text
            start, end = match.span()
            
            # Add the text before the match as a normal run
            if start > last_idx:
                para.add_run(paragraph_text[last_idx:start])

            # Extract the text without the brackets for bold and underline
            text_within_brackets = match.group(1) or match.group(2) or match.group(3)
            if text_within_brackets:
                # Add the extracted text in bold and underline
                run = para.add_run(text_within_brackets)
                run.bold = True
                run.underline = True

            # Update the last index to the end of the current match
            last_idx = end

        # Add any remaining text after the last match
        if last_idx < len(paragraph_text):
            para.add_run(paragraph_text[last_idx:])
"""
def add_bold_underline_to_brackets(cell, paragraph_texts):
    pattern = re.compile(r"\<([^>]*)\>|\[\[([^\]]*)\]\]|\[([^\]]*)\]")

    for idx, paragraph_text in enumerate(paragraph_texts):
        if idx == 0:
            para = cell.paragraphs[0]
            para.clear()
        else:
            para = cell.add_paragraph()
        
        # Check if the paragraph starts with "*" and apply appropriate style
        if paragraph_text.startswith('*'):
            para.style = 'Normal_right'
        else:
            para.style = 'Normal_modified'

        last_idx = 0
        for match in pattern.finditer(paragraph_text):
            start, end = match.span()
            
            if start > last_idx:
                para.add_run(paragraph_text[last_idx:start])

            text_within_brackets = match.group(1) or match.group(2) or match.group(3)
            if text_within_brackets:
                run = para.add_run(text_within_brackets)
                run.bold = True
                run.underline = True

            last_idx = end

        if last_idx < len(paragraph_text):
            para.add_run(paragraph_text[last_idx:])


def 괄호에볼드체하기(cell, paragraph_texts):
    # Define the pattern to match text within < > or [ ]
    pattern = re.compile(r"(\<[^>]*\>|\[[^\]]*\])")

    # Iterate over each paragraph text
    for paragraph_text in paragraph_texts:
        # If it's the first paragraph, use the existing one; otherwise, create a new one
        if paragraph_texts.index(paragraph_text) == 0:
            para = cell.paragraphs[0]
            para.clear()  # Clear the existing paragraph to remove any default text
            para.style = 'Normal_modified'  # Apply the 'Normal_modified' style
        else:
            para = cell.add_paragraph(style='Normal_modified')

        last_idx = 0  # Start index of the last added run
        for match in pattern.finditer(paragraph_text):
            # Add the text before the match
            start, end = match.span()
            para.add_run(paragraph_text[last_idx:start])
            
            # Add the entire matched text (including brackets) in bold and underline
            run = para.add_run(match.group())
            run.bold = True
            #run.underline = True
            
            # Update the last index
            last_idx = end

        # Add any remaining text after the last match
        para.add_run(paragraph_text[last_idx:])



def 특정어구에볼드밑줄더하기(cell, paragraph_texts):
    target_word = '아닌'

    # Iterate over each paragraph text
    for paragraph_text in paragraph_texts:
        # If it's the first paragraph, use the existing one; otherwise, create a new one
        if paragraph_texts.index(paragraph_text) == 0:
            para = cell.paragraphs[0]
            para.clear()  # Clear the existing paragraph to remove any default text
            para.style = 'Normal_modified'  # Apply the 'Normal_modified' style
        else:
            para = cell.add_paragraph(style='Normal_modified')

        # Split the text by the target word
        parts = paragraph_text.split(target_word)

        # Add each part and the target word (if not the last part)
        for i, part in enumerate(parts):
            para.add_run(part)
            if i < len(parts) - 1:  # If not the last part
                run = para.add_run(target_word)
                run.bold = True
                run.underline = True




def remove_brackets(cell, paragraph_texts):
    # Define the pattern to match text within < > or [ ]
    pattern = re.compile(r"\<([^>]*)\>|\[([^\]]*)\]")

    # Iterate over each paragraph text
    for paragraph_text in paragraph_texts:
        # If it's the first paragraph, use the existing one; otherwise, create a new one
        if paragraph_texts.index(paragraph_text) == 0:
            para = cell.paragraphs[0]
            para.clear()  # Clear the existing paragraph to remove any default text
            para.style = 'Normal_modified'  # Apply the 'Normal_modified' style
        else:
            para = cell.add_paragraph(style='Normal_modified')

        last_idx = 0  # Start index of the last added run
        for match in pattern.finditer(paragraph_text):
            # Add the text before the match
            start, end = match.span()
            para.add_run(paragraph_text[last_idx:start])
            # Extract the text without the brackets for bold and underline
            text_within_brackets = match.group(1) or match.group(2)
            # Add the extracted text in bold and underline
            run = para.add_run(text_within_brackets)
            run.bold = False
            run.underline = False
            # Update the last index
            last_idx = end

        # Add any remaining text after the last match
        para.add_run(paragraph_text[last_idx:])





class ExcelToWordConverter:
    def __init__(self, base_path, app_logger, global_text_input_to_extract, 
                 selected_font_for_title, selected_font_for_num, selected_font_for_text, 
                 selected_font_for_exp, size_heading, size_num, size_text, size_exp, 
                 color_heading, color_num, color_text, color_exp, line_text, line_exp, 
                 tagremoval_flag, applyoption_var, book_cover_path=None, text_school_grade=None, text_exam_type=None, text_week=None):
        self.base_path = base_path
        self.app_logger = app_logger
        self.global_text_input_to_extract = global_text_input_to_extract
        self.selected_font_for_title = selected_font_for_title
        self.selected_font_for_num = selected_font_for_num
        self.selected_font_for_text = selected_font_for_text
        self.selected_font_for_exp = selected_font_for_exp
        self.size_heading = size_heading
        self.size_num = size_num
        self.size_text = size_text
        self.size_exp = size_exp
        self.color_heading = color_heading
        self.color_num = color_num
        self.color_text = color_text
        self.color_exp = color_exp
        self.line_text = line_text
        self.line_exp = line_exp
        self.tagremoval_flag = tagremoval_flag
        self.applyoption_var = applyoption_var
        self.book_cover_path = book_cover_path or (self.base_path / "assets" / "bookcovers" / "bookcover01.docx")
        self.close_export_popup = False

        self.text_school_grade = text_school_grade
        self.text_exam_type = text_exam_type
        self.text_week = text_week
        



    # Helper Functions
    
    def add_formatted_text(self, paragraph, text, words_to_format):
        remaining_text = text
        for word in words_to_format:
            if word in remaining_text:
                parts = remaining_text.split(word, 1)
                if parts[0]:
                    paragraph.add_run(parts[0])
                run = paragraph.add_run(word)
                run.bold = True
                run.underline = True
                remaining_text = parts[1]
            else:
                paragraph.add_run(remaining_text)
                remaining_text = ''
        if remaining_text:
            paragraph.add_run(remaining_text)    
    
    def 테이블없는본문_괄호에볼드밑줄더하기(self, paragraph, text):
        remaining_text = text
        while '[[' in remaining_text and ']]' in remaining_text:
            before_brackets, after_brackets = remaining_text.split('[[' , 1)
            inside_brackets, remaining_text = after_brackets.split(']]', 1)
            
            # Add the text before the brackets normally
            if before_brackets:
                paragraph.add_run(before_brackets)
            
            # Add the text inside the brackets with bold and underline
            run = paragraph.add_run(inside_brackets)
            run.bold = True
            run.underline = True
        
        # Finally, add any remaining text that is left after processing the brackets
        if remaining_text:
            paragraph.add_run(remaining_text)


    def 테이블없는본문_영작단어수에볼드밑줄더하기(self, paragraph, text):
        # Define the regex pattern to match one or two digits followed by '단어'
        pattern = re.compile(r'\d{1,2}단어')
        
        # Initialize the starting index
        last_end = 0
        
        # Iterate over all matches in the text
        for match in pattern.finditer(text):
            start, end = match.span()
            
            # Add the text before the matched pattern normally
            if start > last_end:
                normal_text = text[last_end:start]
                if normal_text:
                    paragraph.add_run(normal_text)
            
            # Add the matched text with bold and underline formatting
            matched_text = text[start:end]
            if matched_text:
                run = paragraph.add_run(matched_text)
                run.bold = True
                run.underline = True
            
            # Update the last_end to the end of the current match
            last_end = end
        
        # Add any remaining text after the last match
        if last_end < len(text):
            remaining_text = text[last_end:]
            if remaining_text:
                paragraph.add_run(remaining_text)


    def hex_to_rgb(self, hex_color):
        hex_color = hex_color.lstrip('#')  # Remove '#' at the start if present
        lv = len(hex_color)
        return tuple(int(hex_color[i:i + lv // 3], 16) for i in range(0, lv, lv // 3))

    def set_cell_paragraph_style(self, cell, style_name):
        for paragraph in cell.paragraphs:
            paragraph.style = style_name

    def set_cell_font_hdr(self, cell, font_name, font_size):
        font_size += 1
        for paragraph in cell.paragraphs:
            for run in paragraph.runs:
                run.font.name = font_name
                run._element.rPr.rFonts.set(qn('w:eastAsia'), font_name)  # Korean font setting
                run.font.size = Pt(font_size)
                run.font.bold = True
            paragraph.alignment = WD_PARAGRAPH_ALIGNMENT.CENTER
            paragraph.paragraph_format.line_spacing_rule = WD_LINE_SPACING.MULTIPLE
            paragraph.paragraph_format.line_spacing = 1.1

    def set_cell_border(self, cell, **kwargs):
        tc = cell._tc
        tcPr = tc.get_or_add_tcPr()

        for edge in ('top', 'bottom', 'left', 'right', 'insideH', 'insideV'):
            edge_data = kwargs.get(edge)
            if edge_data:
                tag = f'w:{edge}'
                element = OxmlElement(tag)
                element.set(qn('w:val'), edge_data.get('val', 'single'))
                element.set(qn('w:sz'), str(edge_data.get('sz', 12)))
                element.set(qn('w:space'), str(edge_data.get('space', 0)))
                element.set(qn('w:color'), edge_data.get('color', 'auto'))
                tcPr.append(element)

    def set_table_cell_margins(self, table, top=0, bottom=0, left=0, right=0):
        for row in table.rows:
            for cell in row.cells:
                self.set_cell_margins(cell, top, bottom, left, right)

    def set_cell_margins(self, cell, top=0, bottom=0, left=0, right=0):
        top_dxa = str(int(top * 20))
        bottom_dxa = str(int(bottom * 20))
        left_dxa = str(int(left * 20))
        right_dxa = str(int(right * 20))

        tcPr = cell._element.get_or_add_tcPr()
        tcMar = OxmlElement('w:tcMar')

        for mar, value in [('top', top_dxa), ('bottom', bottom_dxa),
                          ('left', left_dxa), ('right', right_dxa)]:
            mar_element = OxmlElement(f'w:{mar}')
            mar_element.set(qn('w:w'), value)
            mar_element.set(qn('w:type'), "dxa")
            tcMar.append(mar_element)

        tcPr.append(tcMar)

    def count_words(self, text):
        return len(text.split())

    def add_page_break(self, doc):
        doc.add_paragraph(style='Normal_modified').add_run().add_break(WD_BREAK.PAGE)

    def is_section_title(self, para_text):
        pattern = r"^\d+\.\s(어휘|내용|밑줄의미|함축의미|빈칸추론|추론|순서|순서2단계|삽입|주제|제목|요지|연결어|어법|빈칸암기|동반의어|영영정의|요약|영작|동형문제|커스텀|실전모의고사|기출문제세트).*"
        return bool(re.match(pattern, para_text))

    def is_answer_and_explanation(self, para_text):
        return para_text.strip() == "정답 및 해설"

    def fast_answer(self, para_text):
        return para_text.strip() == "빠른 정답 찾기"


    def is_answer_heading(self, para_text):
        return para_text.endswith("정답")

    def define_styles(self, doc):

        # Define 'Heading 1' style
        heading1_style = doc.styles['Heading 1']
        heading1_style.font.name = self.selected_font_for_title
        heading1_style._element.rPr.rFonts.set(qn('w:eastAsia'), self.selected_font_for_title)
        heading1_style.font.size = Pt(self.size_heading)
        heading1_style.paragraph_format.line_spacing_rule = WD_LINE_SPACING.MULTIPLE
        heading1_style.paragraph_format.line_spacing = 1.2
        heading1_style.paragraph_format.space_before = Pt(0)
        heading1_style.paragraph_format.space_after = Pt(0)
        heading1_style.paragraph_format.alignment = WD_PARAGRAPH_ALIGNMENT.JUSTIFY

        rgb_color_heading = self.hex_to_rgb(self.color_heading)
        heading1_style.font.color.rgb = RGBColor(*rgb_color_heading)
        heading1_style.font.bold = True

        # Define 'Heading_ans' style
        heading1ans_style = doc.styles.add_style('Heading_ans', WD_STYLE_TYPE.PARAGRAPH)
        heading1ans_style.base_style = doc.styles['Heading 2']
        heading1ans_style.font.name = self.selected_font_for_title
        heading1ans_style._element.rPr.rFonts.set(qn('w:eastAsia'), self.selected_font_for_title)
        heading1ans_style.font.size = Pt(self.size_heading)
        heading1ans_style.paragraph_format.space_before = Pt(0)
        heading1ans_style.paragraph_format.space_after = Pt(0)
        heading1ans_style.paragraph_format.line_spacing_rule = WD_LINE_SPACING.MULTIPLE
        heading1ans_style.paragraph_format.line_spacing = 1.2
        heading1ans_style.paragraph_format.alignment = WD_PARAGRAPH_ALIGNMENT.JUSTIFY
        heading1ans_style.font.color.rgb = RGBColor(*rgb_color_heading)
        heading1ans_style.font.bold = True

        # Define 'Heading_num' style
        heading_num_style = doc.styles.add_style('Heading_num', WD_STYLE_TYPE.PARAGRAPH)
        heading_num_style.base_style = doc.styles['Heading 2']
        heading_num_style.font.name = self.selected_font_for_num
        heading_num_style._element.rPr.rFonts.set(qn('w:eastAsia'), self.selected_font_for_num)
        heading_num_style.font.size = Pt(self.size_num)
        heading_num_style.paragraph_format.space_before = Pt(0)
        heading_num_style.paragraph_format.space_after = Pt(0)
        heading_num_style.paragraph_format.line_spacing_rule = WD_LINE_SPACING.MULTIPLE
        heading_num_style.paragraph_format.line_spacing = 1.2
        rgb_color_num = self.hex_to_rgb(self.color_num)
        heading_num_style.font.color.rgb = RGBColor(*rgb_color_num)
        heading_num_style.font.bold = True

        # Define 'Normal' style
        normal_style = doc.styles['Normal']
        normal_style.font.name = '나눔고딕'
        normal_style._element.rPr.rFonts.set(qn('w:eastAsia'), '나눔고딕')
        normal_style.font.size = Pt(8)
        normal_style.paragraph_format.line_spacing_rule = WD_LINE_SPACING.MULTIPLE
        normal_style.paragraph_format.line_spacing = 1.2
        normal_style.paragraph_format.space_before = Pt(0)
        normal_style.paragraph_format.space_after = Pt(0)
        normal_style.paragraph_format.alignment = WD_PARAGRAPH_ALIGNMENT.JUSTIFY

        # Define 'Normal_modified' style
        normal_style_modified = doc.styles.add_style('Normal_modified', WD_STYLE_TYPE.PARAGRAPH)
        normal_style_modified.base_style = doc.styles['Normal']
        normal_style_modified.font.name = self.selected_font_for_text
        normal_style_modified._element.rPr.rFonts.set(qn('w:eastAsia'), self.selected_font_for_text)
        normal_style_modified.font.size = Pt(self.size_text)
        normal_style_modified.paragraph_format.line_spacing_rule = WD_LINE_SPACING.MULTIPLE
        normal_style_modified.paragraph_format.line_spacing = self.line_text
        normal_style_modified.paragraph_format.space_before = Pt(0)
        normal_style_modified.paragraph_format.space_after = Pt(0)
        normal_style_modified.paragraph_format.alignment = WD_PARAGRAPH_ALIGNMENT.JUSTIFY
        rgb_color_text = self.hex_to_rgb(self.color_text)
        normal_style_modified.font.color.rgb = RGBColor(*rgb_color_text)

        # Define 'Normal_left' style
        normal_style_left = doc.styles.add_style('Normal_left', WD_STYLE_TYPE.PARAGRAPH)
        normal_style_left.base_style = doc.styles['Normal']
        normal_style_left.font.name = self.selected_font_for_text
        normal_style_left._element.rPr.rFonts.set(qn('w:eastAsia'), self.selected_font_for_text)
        normal_style_left.font.size = Pt(self.size_text)
        normal_style_left.paragraph_format.line_spacing_rule = WD_LINE_SPACING.MULTIPLE
        normal_style_left.paragraph_format.line_spacing = self.line_text
        normal_style_left.paragraph_format.space_before = Pt(0)
        normal_style_left.paragraph_format.space_after = Pt(0)
        normal_style_left.paragraph_format.alignment = WD_PARAGRAPH_ALIGNMENT.LEFT
        normal_style_left.font.color.rgb = RGBColor(*rgb_color_text)

        # Define 'Normal_right' style (주석어휘)
        normal_style_right = doc.styles.add_style('Normal_right', WD_STYLE_TYPE.PARAGRAPH)
        normal_style_right.base_style = doc.styles['Normal']
        normal_style_right.font.name = self.selected_font_for_text
        normal_style_right._element.rPr.rFonts.set(qn('w:eastAsia'), self.selected_font_for_text)
        normal_style_right.font.size = Pt(self.size_text - 1)
        normal_style_right.paragraph_format.line_spacing_rule = WD_LINE_SPACING.MULTIPLE
        normal_style_right.paragraph_format.line_spacing = self.line_text
        normal_style_right.paragraph_format.space_before = Pt(0)
        normal_style_right.paragraph_format.space_after = Pt(0)
        normal_style_right.paragraph_format.alignment = WD_PARAGRAPH_ALIGNMENT.RIGHT
        normal_style_right.font.color.rgb = RGBColor(*rgb_color_text)

        # Define 'Normal_exp' style
        normal_style_explanation = doc.styles.add_style('Normal_exp', WD_STYLE_TYPE.PARAGRAPH)
        normal_style_explanation.base_style = doc.styles['Normal']
        normal_style_explanation.font.name = self.selected_font_for_exp
        normal_style_explanation._element.rPr.rFonts.set(qn('w:eastAsia'), self.selected_font_for_exp)
        normal_style_explanation.font.size = Pt(self.size_exp)
        normal_style_explanation.paragraph_format.line_spacing_rule = WD_LINE_SPACING.MULTIPLE
        normal_style_explanation.paragraph_format.line_spacing = self.line_exp
        normal_style_explanation.paragraph_format.space_before = Pt(0)
        normal_style_explanation.paragraph_format.space_after = Pt(0)
        normal_style_explanation.paragraph_format.alignment = WD_PARAGRAPH_ALIGNMENT.LEFT
        rgb_color_exp = self.hex_to_rgb(self.color_exp)
        normal_style_explanation.font.color.rgb = RGBColor(*rgb_color_exp)



    """
    def merge_docs(self, cover_path, content_path):
        cover_doc = Document(cover_path)
        content_doc = Document(content_path)

        # Set document margins
        section = cover_doc.sections[0]
        section.top_margin = Cm(1.5)
        section.bottom_margin = Cm(1.6)
        section.left_margin = Cm(1.9)
        section.right_margin = Cm(1.9)

        # Define styles for cover_doc
        self.define_styles(cover_doc)

        # Append content_doc elements to cover_doc
        for element in content_doc.element.body:
            if not element.tag.endswith('sectPr'):
                cover_doc.element.body.append(element)

        # Apply styles to paragraphs
        for para in cover_doc.paragraphs:
            if self.is_answer_heading(para.text):
                para.style = cover_doc.styles["Heading_ans"]
            elif self.is_section_title(para.text) or self.is_answer_and_explanation(para.text):
                para.style = cover_doc.styles["Heading 1"]

        # Remove blank lines before section titles
        for i, para in enumerate(cover_doc.paragraphs):
            if i > 0 and self.is_section_title(para.text) and not cover_doc.paragraphs[i - 1].text.strip():
                p = cover_doc.paragraphs[i - 1]._element
                p.getparent().remove(p)

        # Save the merged document
        cover_doc.save(content_path)
    """

        
    def handle_empty_export(self):
        error_dialog = Toplevel()
        error_dialog.title("오류")
        error_dialog.geometry("500x150")
        error_dialog.grab_set()
        error_dialog.focus_set()
        error_dialog.grid_rowconfigure(0, weight=1)
        error_dialog.grid_columnconfigure(0, weight=1)

        CTkLabel(
            error_dialog, 
            fg_color="white", 
            width=300, 
            height=120, 
            text="-추출할 지문이 없습니다.\n'추출할 지문번호 입력'창에 추출할 지문번호가 올바르게 입력되어 있는지 확인하세요.", 
            text_color="black"
        ).grid(row=0, column=0, sticky="nsew")

        button_frame = CTkFrame(error_dialog, fg_color="white", width=300, height=30, corner_radius=0)
        button_frame.grid(row=1, column=0, sticky="swe")
        button_frame.grid_columnconfigure(0, weight=1)

        def on_ok():
            self.close_export_popup = True
            error_dialog.grab_release()
            error_dialog.destroy()

        ok_button = CTkButton(
            button_frame, 
            border_width=2, 
            border_color="gray", 
            text="OK", 
            text_color="black", 
            width=70, 
            height=30, 
            command=on_ok, 
            fg_color="#FEF9E0", 
            hover_color="#DDA15C"
        )
        ok_button.grid(row=0, column=0, padx=10, pady=10)




    def excel_to_word(self, excel_file, sheet_name, output_doc, last_state, rows_order):
        """
        Converts specified sheets from an Excel file to a formatted Word document.

        Parameters:
            excel_file (str): Path to the Excel file.
            sheet_name (str): Name of the sheet to process.
            output_doc (str): Path to save the generated Word document.
            last_state (dict): Dictionary indicating which sheets to process.
            rows_order (list): List indicating the order of processing sheets.
        """
        # Debug statement
        #print("FUNCTION_MAP inside class:", FUNCTION_MAP)

        if not self.book_cover_path:
            self.book_cover_path = self.base_path / "assets" / "bookcovers" / "bookcover01.docx"

        # Load the workbook and sheets
        try:
            wb = load_workbook(filename=excel_file, read_only=True)
        except Exception as e:
            self.app_logger.error(f"Failed to load workbook '{excel_file}': {e}")
            return

        # Attempt to access the '대기열' sheet; handle if it doesn't exist
        try:
            waiting_sheet = wb["대기열"]
        except KeyError:
            self.app_logger.error(f"Sheet '대기열' does not exist in the workbook.")
            return

        print("self.book_cover_path:")
        print(self.book_cover_path)

        # Create a new Word document
        doc = Document()
        self.define_styles(doc)

        # Set document margins
        section = doc.sections[0]
        section.top_margin = Cm(1.5)
        section.bottom_margin = Cm(1.6)
        section.left_margin = Cm(1.9)
        section.right_margin = Cm(1.9)

        sections_dict = defaultdict(list)
        section_number = 1

        # Process each row_key in the specified order

        for row_key in rows_order:
            normal_key = f"{row_key}_Normal"
            hard_key = f"{row_key}_Hard"
            process_normal = last_state.get(normal_key, False)
            process_hard = last_state.get(hard_key, False)
            process_mock_test = last_state.get(row_key, False)
            process_past_test = last_state.get(row_key, False)

            for sheet_suffix, process in [('Normal', process_normal), ('Hard', process_hard)]:
                if process:
                    # Determine the sheet type based on the suffix
                    if "실전모의고사" in row_key or "기출문제세트" in row_key:
                        sheet_type = f"{row_key}"
                    else:
                        sheet_type = f"{row_key}_{sheet_suffix}"

                    # Check if the sheet exists in the workbook
                    if sheet_type in wb.sheetnames:
                        print(f"Processing sheet_type: {sheet_type}")
                        extracting_sheet = wb[sheet_type]

                        # Fetch text from global_text_input_to_extract and split into lines
                        text_content = self.global_text_input_to_extract.get("1.0", "end-1c")
                        lines = [line.strip() for line in text_content.split("\n") if line.strip()]

                        # Initialize an empty list to collect unique passage IDs while maintaining order
                        추출할지문번호 = []

                        # Iterate through each line, treating it as a passage ID
                        for line in lines:
                            if line:
                                stripped_passage_id = line.strip()
                                if stripped_passage_id not in 추출할지문번호:
                                    추출할지문번호.append(stripped_passage_id)
                                else:
                                    self.app_logger.info(f"●[오류] 지문번호 중복: {stripped_passage_id}\n첫 번째 지문만 추출됩니다.")

                        current_word_count = 0
                        section_title = f"{section_number}. {sheet_type}"

                        # Remove difficulty tag if flag is set
                        if self.tagremoval_flag == 1:
                            modified_sheet_type = sheet_type.split('_')[0].strip()
                            section_title = f"{section_number}. {modified_sheet_type}"
                            doc.add_paragraph(section_title, style='Heading 1')
                        else:
                            doc.add_paragraph(section_title, style='Heading 1')

                        # Create a mapping from passage ID to row data in extracting_sheet
                        passage_id_to_row_data = {}
                        for row in extracting_sheet.iter_rows(min_row=1, values_only=True):
                            if row[0] is not None:
                                if len(row) >= 3:
                                    passage_id = row[0].strip()
                                    passage_id_to_row_data[passage_id] = row[:5]

                        #print(f"passage_id_to_row_data: {passage_id_to_row_data}")

                        # Log missing passages
                        #for 지문번호 in 추출할지문번호:
                        #    if 지문번호 not in passage_id_to_row_data:
                        #        self.app_logger.error(f"[에러] '{sheet_type}' 유형에 '{지문번호}' 지문이 없음. (추출할 유형과 지문번호를 다시 확인해보세요)")

                        # Iterate through 추출할지문번호 in the order they appear in waiting_sheet
                        for passage_id in 추출할지문번호:
                            if passage_id not in passage_id_to_row_data:
                                self.app_logger.error(f"[에러★] '{sheet_type}' 유형에 '{passage_id}' 지문이 없음. (추출할 유형과 지문번호를 다시 확인해보세요)")
                                continue

                            row_data = passage_id_to_row_data[passage_id]
                            if len(row_data) < 5:
                                row_data += ("",) * (5 - len(row_data))
                            title, passage, result, answer_part, explanation_part = row_data[:5]

                            if title:
                                title = title.strip()

                                # Skip certain results
                                if title in 추출할지문번호 and result == "연결어 없음":
                                    self.app_logger.info(f"{title}: 연결어가 없는 지문이라 추출하지 않습니다.")
                                    continue
                                if title in 추출할지문번호 and result == "문제 미출제 (문장 수 부족)":
                                    self.app_logger.info(f"{title}: 문장 수가 부족하여 추출하지 않습니다.")
                                    continue
                                if title in 추출할지문번호 and (result is None or result == ""):
                                    self.app_logger.info(f"{title}: 출제된 문제가 없습니다.")
                                    continue
                                elif title in 추출할지문번호 and result.startswith("Error"):
                                    self.app_logger.info(f"{sheet_type} {title}: 출제된 문제에 에러가 있어서 추출하지 않습니다. 해당 유형/해당 지문번호의 '문제' 부분에 Error라고 써 있는 부분을 문제를 더블클릭 후 '문제 재출제'를 클릭하여 다시 출제하세요.")
                                    continue
                                elif title in 추출할지문번호 and result is not None:

                                    # Determine function name
                                    #function_name = self.determine_function_name(sheet_type)
                                    function_name = sheet_type.replace('_Normal', '_편집').replace('_Hard', '_편집').replace("내용일치(한)", "내용일치한") 

                                    # Check if the function exists in FUNCTION_MAP
                                    if function_name in FUNCTION_MAP:

                                        para_title = doc.add_paragraph(style='Heading_num')  # Add the title as a heading
                                        para_title.add_run(str(title))


                                        # Call the function using process_question_part and unpack all returned values
                                        title, passage, question_line, question_part, answer_part, explanation_part, cell, table = self.process_question_part(
                                            doc, function_name, title, passage, result, answer_part, explanation_part
                                        )

 

                                        #if '연결어' not in function_name and '빈칸암기' not in function_name:
                                        #    title, passage, question_part, answer_part, explanation_part = globals()[function_name](title, passage, result, answer_part, explanation_part)


                                        #print(f"function_name: {function_name}")
                                        #print(f"passage: {passage}")
                                        #print(f"question_part: {question_part}")

                                        # 서식 편집
                                        if '동반의어_편집' in function_name:
                                            data = parse_data(question_part)
                                            insert_data_into_table(doc, data)
                                            cell = None
                                            answer_part = " "
                                            explanation_part = " "

                                        elif '영영정의_편집' in function_name:
                                            data = parse_data(question_part)
                                            insert_data_into_table_영영정의(doc, data)
                                            cell = None
                                            answer_part = " "
                                            explanation_part = " "

                                        elif '커스텀' in function_name or '요약문' in function_name:
                                            for paragraph in question_part.split('\n'):
                                                doc.add_paragraph(paragraph, style='Normal_modified')
                                            cell = None
                                            answer_part = " "
                                            explanation_part = " "

                                        elif '어휘1단계' in function_name or '어휘2단계' in function_name or '어법1단계' in function_name:
                                            table = doc.add_table(rows=1, cols=1)
                                            cell = table.cell(0, 0)
                                            cell.text = ""
                                            paragraphs = question_part.split('\n')
                                            add_bold_brackets_to_cell_paragraph(cell, paragraphs)  # Add bold to brackets

                                        elif '어휘3단계' in function_name or '어법2단계' in function_name:
                                            table = doc.add_table(rows=1, cols=1)
                                            cell = table.cell(0, 0)
                                            cell.text = ""
                                            paragraphs = passage.split('\n')
                                            add_bold_underline_to_brackets(cell, paragraphs)  # Add bold + underline to brackets
                                            for paragraph in question_part.split('\n'):
                                                doc.add_paragraph(paragraph, style='Normal_modified')


                                        elif '밑줄의미' in function_name or '함축의미' in function_name:
                                            #문제를 요약문과 선지로 쪼개기
                                            split_point = question_part.find("①")
                                            문제라인파트 = question_part[:split_point].strip()
                                            문제나머지파트 = question_part[split_point:].strip()

                                            #문제라인 추가
                                            if '밑줄의미' in function_name:
                                                question_line_part = doc.add_paragraph(style='Normal_modified')
                                                words_to_format = ["않은"]
                                                self.add_formatted_text(question_line_part, 문제라인파트, words_to_format)
                                            else:
                                                question_line_part = doc.add_paragraph(style='Normal_modified')
                                                self.테이블없는본문_괄호에볼드밑줄더하기(question_line_part, 문제라인파트)

                                            #지문 추가
                                            table = doc.add_table(rows=1, cols=1)
                                            cell = table.cell(0, 0)
                                            cell.text = ""
                                            paragraphs = passage.split('\n')
                                            add_bold_underline_to_brackets(cell, paragraphs)  # Add bold + underline to brackets


                                            #문제 나머지파트 추가
                                            for paragraph in 문제나머지파트.split('\n'):
                                                doc.add_paragraph(paragraph, style='Normal_modified')




                                        elif '빈칸추론' in function_name:
                                            table = doc.add_table(rows=1, cols=1)
                                            cell = table.cell(0, 0)
                                            cell.text = ""
                                            paragraphs = passage.split('\n')
                                            remove_brackets(cell, paragraphs)
                                            for paragraph in question_part.split('\n'):
                                                doc.add_paragraph(paragraph, style='Normal_modified')


                                        elif any(keyword in function_name for keyword in ['내용일치', '내용일치(한)', '추론', '순서', '삽입', '주제', '제목', '요지', '어휘종합', '내용종합']):
                                            table = doc.add_table(rows=1, cols=1)
                                            cell = table.cell(0, 0)
                                            cell.text = ""
                                            paragraphs = passage.split('\n')
                                            add_bold_underline_to_brackets(cell, paragraphs)
                                            for paragraph in question_part.split('\n'):
                                                doc.add_paragraph(paragraph, style='Normal_modified')



    

                                        elif '요약_편집' in function_name:

                                            question_part = question_part.replace("윗글의 내용을 한 문장으로 요약하고자 한다. 빈칸 (A), (B)에 들어갈 말로 가장 적절한 것은?\n", "")

                                            #문제 라인 입력
                                            question_line = "다음 글의 내용을 한 문장으로 요약하고자 한다. 빈칸 (A), (B)에 들어갈 말로 가장 적절한 것은?"
                                            para_question_line = doc.add_paragraph(question_line, style='Normal_modified')
                                            para_question_line.alignment = WD_ALIGN_PARAGRAPH.LEFT


                                            #지문 테이블에 입력
                                            table = doc.add_table(rows=1, cols=1)
                                            cell = table.cell(0, 0)
                                            cell.text = ""

                                            paragraphs = passage.split('\n')
                                            add_bold_underline_to_brackets(cell, paragraphs)

                                            #요약문 기호 입력
                                            요약문기호 = "⇩"
                                            para = doc.add_paragraph(요약문기호, style='Normal_modified')
                                            para.alignment = WD_ALIGN_PARAGRAPH.CENTER


                                            #문제를 요약문과 선지로 쪼개기
                                            split_point = question_part.find("(A) -")
                                            요약문파트 = question_part[:split_point].strip()
                                            선지파트 = question_part[split_point:].strip()

                                            
                                            #요약문 파트 괄호 제거하고 표에 넣기
                                            table2 = doc.add_table(rows=1, cols=1)
                                            cell2 = table2.cell(0, 0)
                                            cell2.text = ""

                                            요약문파트_paragraphs = 요약문파트.split('\n')
                                            remove_brackets(cell2, 요약문파트_paragraphs)
                                            

                                            # 선지파트 탭 적용 후 넣기
                                            선지파트 = 선지파트.replace("(A) - (B)", "          (A)\t\t       (B)")
                                            선지파트 = 선지파트.replace(" -", "\t...\t")

                                            for paragraph_text in 선지파트.split('\n'):
                                                paragraph = doc.add_paragraph(paragraph_text, style='Normal_modified')  # Now paragraph is a Paragraph object

                                                # Set custom tab stops at desired intervals for the options paragraph
                                                tab_stops = paragraph.paragraph_format.tab_stops
                                                for tab_pos in range(1, 2):  # Assuming we want a tab stop between each option
                                                    tab_stops.add_tab_stop(Inches(1.1 * tab_pos))

                                        
                                            self.set_cell_border(cell2, top={"sz": 8, "val": "single", "color": "auto"},
                                                            bottom={"sz": 8, "val": "single", "color": "auto"},
                                                            left={"sz": 8, "val": "single", "color": "auto"},
                                                            right={"sz": 8, "val": "single", "color": "auto"})
                                        
                                            self.set_table_cell_margins(table2, top=5, bottom=5, left=7, right=7)
                                        


                                        elif '영영정의문제' in function_name:
                                            table = doc.add_table(rows=1, cols=1)
                                            cell = table.cell(0, 0)
                                            cell.text = ""
                                            paragraphs = passage.split('\n')
                                            괄호에볼드체하기(cell, paragraphs)
                                            for paragraph in question_part.split('\n'):
                                                doc.add_paragraph(paragraph, style='Normal_modified')



                                        elif '영작1' in function_name:
                                                      
                                            

                                            # Find and capture the "문제파트"
                                            문제파트 = ""
                                            # Use regex to match the entire problem part before the actual content
                                            match = re.search(r"다음의 밑줄 친.*?\)", question_part)
                                            if match:
                                                문제파트 = match.group(0)  # Capture the entire "문제파트"

                                            # Remove the "문제파트" from the original text
                                            question_part = question_part.replace(문제파트, "").strip()

                                            # Now safely split the remaining text at the first "[보기]" to get 지문파트 and 보기파트
                                            if '[보기]' in question_part:
                                                지문파트, 보기파트 = question_part.split('[보기]', 1)  # Use 1 to ensure only the first occurrence is used
                                                지문파트 = 지문파트.strip()  # Clean up any surrounding whitespace
                                                보기파트 = '[보기] ' + 보기파트.strip()  # Add [보기] back and clean whitespace
                                            else:
                                                # Handle cases where "[보기]" might not be present
                                                지문파트 = question_part.strip()
                                                보기파트 = ""                                            


                                            # 문제라인
                                            question_line_part = doc.add_paragraph(style='Normal_modified')
                                            self.테이블없는본문_영작단어수에볼드밑줄더하기(question_line_part, 문제파트)
                                            


                                            # 지문파트
                                            table = doc.add_table(rows=1, cols=1)
                                            cell = table.cell(0, 0)
                                            cell.text = ""
                                            paragraphs = 지문파트.split('\n')
                                            add_bold_underline_to_brackets(cell, paragraphs)  # Add bold + underline to brackets


                                            # 보기파트
                                            보기파트 = 보기파트 + "\n\n → ____________________________________________________________________________________________"

                                            for paragraph in 보기파트.split('\n'):
                                                doc.add_paragraph(paragraph, style='Normal_modified')



                                        elif '영작2' in function_name:
                                                                                               


                                            # Find and capture the "문제파트"
                                            문제파트 = ""
                                            # Use regex to match the entire problem part before the actual content
                                            match = re.search(r"\[보기\]에.*?\)", question_part)
                                            if match:
                                                문제파트 = match.group(0)  # Capture the entire "문제파트"

                                            # Remove the "문제파트" from the original text
                                            question_part = question_part.replace(문제파트, "").strip()

                                            # Now safely split the remaining text at the first "[보기]" to get 지문파트 and 보기파트
                                            if '[보기]' in question_part:
                                                지문파트, 보기파트 = question_part.split('[보기]', 1)  # Use 1 to ensure only the first occurrence is used
                                                지문파트 = 지문파트.strip()  # Clean up any surrounding whitespace
                                                보기파트 = '[보기] ' + 보기파트.strip()  # Add [보기] back and clean whitespace
                                            else:
                                                # Handle cases where "[보기]" might not be present
                                                지문파트 = question_part.strip()
                                                보기파트 = ""                                            


                                            # 문제라인
                                            question_line_part = doc.add_paragraph(style='Normal_modified')
                                            self.테이블없는본문_영작단어수에볼드밑줄더하기(question_line_part, 문제파트)
                                            


                                            # 지문파트
                                            table = doc.add_table(rows=1, cols=1)
                                            cell = table.cell(0, 0)
                                            cell.text = ""
                                            paragraphs = 지문파트.split('\n')
                                            add_bold_underline_to_brackets(cell, paragraphs)  # Add bold + underline to brackets


                                            # 보기파트
                                            보기파트 = 보기파트 + "\n\n → ____________________________________________________________________________________________"

                                            for paragraph in 보기파트.split('\n'):
                                                doc.add_paragraph(paragraph, style='Normal_modified')

                                        elif '동형문제' in function_name:

                                            #지문 추가
                                            table = doc.add_table(rows=1, cols=1)
                                            cell = table.cell(0, 0)
                                            cell.text = ""
                                            paragraphs = passage.split('\n')
                                            add_bold_underline_to_brackets(cell, paragraphs)  # Add bold + underline to brackets

                                            #문제 추가
                                            for paragraph in question_part.split('\n'):
                                                doc.add_paragraph(paragraph, style='Normal_modified')


                                        #연결어랑 빈칸암기는 편집함수(editing_funcions.py) 내에서 처리함.


                                        if cell is not None:
                                            self.set_cell_border(cell, top={"sz": 8, "val": "single", "color": "auto"},
                                                            bottom={"sz": 8, "val": "single", "color": "auto"},
                                                            left={"sz": 8, "val": "single", "color": "auto"},
                                                            right={"sz": 8, "val": "single", "color": "auto"})
                                            if table is not None:
                                                self.set_table_cell_margins(table, top=5, bottom=5, left=7, right=7)

                                        doc.add_paragraph(style='Normal_modified') #문제 끝에 빈 줄 하나 추가



                                        


                                        # Check if answer_part is empty after processing
                                        if not answer_part and "영작" not in sheet_type:
                                            self.show_error_and_log(
                                                f"오류: '{sheet_type}' 유형의 '{title}' 지문에서 문제 오류가 발견되었습니다.\n"
                                                f"(오류내용: '정답'칸이 비어 있음)\n해당 문제를 삭제하거나 재출제하세요.★"
                                            )
                                            return
                                        sections_dict[section_title].append((title, answer_part, explanation_part))
                                    else:
                                        self.app_logger.error(f"Function {function_name} not found in FUNCTION_MAP.")
                                        continue

                        section_number += 1  # Increment the section number for the next section
                        self.add_page_break(doc)                                    

            # Handle Mock Test
            if "실전모의고사" in row_key and process_mock_test:
                #self.app_logger.info("실전모의고사 문제지 추출 시작!")
                #self.app_logger.info(f"sheet_type: {row_key}")

                self.handle_mock_test_문제지(wb, row_key, doc, section_number, sections_dict)
                section_number += 1
                self.add_page_break(doc)

            elif "기출문제세트" in row_key and process_past_test:
                self.handle_past_test_문제지(wb, row_key, doc, section_number, sections_dict)
                section_number += 1
                self.add_page_break(doc)


        ############################## $정답 $해설 ########################################

        # After processing all sections, add the answer and explanation section
        print("해설지 시작")
        doc.add_paragraph("\n\n\n\n\n")  # Add space before the '정답 및 해설' section
        paragraph = doc.add_paragraph("정답 및 해설", style='Heading 1')
        paragraph.alignment = WD_PARAGRAPH_ALIGNMENT.CENTER

        # Access the first run of the paragraph to set the font size
        if paragraph.runs:
            paragraph.runs[0].font.size = Pt(20)
        else:
            # If there are no runs, add a new run and set its properties
            run = paragraph.add_run("정답 및 해설")
            run.font.size = Pt(20)

        doc.add_page_break()

        # Attempt to retrieve section titles
        try:
            section_titles = list(sections_dict.keys())  # Get all the section titles
            #self.app_logger.info("Section Titles: %s", section_titles)
        except IndexError:
            # Handle the error or pass if there's nothing specific to do
            messagebox.showerror("오류", "●추출할 지문이 없습니다. '추출할 지문번호 입력' 창에 지문번호가 올바르게 입력되어 있는지 확인하세요.")
            self.app_logger.info("●[오류] 추출할 지문이 없습니다.")
            return
        except Exception as e:
            # It's good practice to catch other potential errors
            self.app_logger.info(f"An unexpected error occurred: {e}")
            return

        # Determine the last section title, excluding specific keywords
        try:
            filtered_section_titles = [
                title for title in section_titles 
                if "동반의어_Normal" not in title and 
                "영영정의_Normal" not in title and 
                "커스텀" not in title and 
                "요약문" not in title and title != "동반의어" and title != "영영정의"
            ]
            if filtered_section_titles:
                last_section_title = filtered_section_titles[-1]  # Get the last section title
            else:
                last_section_title = section_titles[-1]  # Fallback to the last section title
        except IndexError:
            # Handle the error or notify the user
            def nothing_to_export():
                error_dialog = Toplevel()
                error_dialog.title("오류")
                error_dialog.geometry("500x150")
                error_dialog.grab_set()  # Make the dialog modal
                error_dialog.focus_set()  # Make the dialog modal
                error_dialog.grid_rowconfigure(0, weight=1)
                error_dialog.grid_columnconfigure(0, weight=1)

                CTkLabel(
                    error_dialog, 
                    fg_color="white", 
                    width=300, 
                    height=120, 
                    text="-추출할 지문이 없습니다.\n'추출할 지문번호 입력'창에 추출할 지문번호가 올바르게 입력되어 있는지 확인하세요.", 
                    text_color="black"
                ).grid(row=0, column=0, sticky="nsew")

                # Frame for buttons
                button_frame = CTkFrame(error_dialog, fg_color="white", width=300, height=30, corner_radius=0)
                button_frame.grid(row=1, column=0, sticky="swe")
                button_frame.grid_columnconfigure(0, weight=1)

                # "OK" button action
                def on_ok():
                    self.close_export_popup = True
                    error_dialog.grab_release()
                    error_dialog.destroy()

                # Create "OK" button
                ok_button = CTkButton(
                    button_frame, 
                    border_width=2, 
                    border_color="gray", 
                    text="OK", 
                    text_color="black", 
                    width=70, 
                    height=30, 
                    command=on_ok, 
                    fg_color="#FEF9E0", 
                    hover_color="#DDA15C"
                )
                ok_button.grid(row=0, column=0, padx=10, pady=10)

            self.close_export_popup = False
            nothing_to_export()
            return
        except Exception as e:
            # It's good practice to catch other potential errors
            self.app_logger.info(f"An unexpected error occurred: {e}")
            return

        # Handle sections_dict for answers and explanations
        self.process_sections_dict_해설지(doc, sections_dict, last_section_title)















        # After processing all sections, add the answer and explanation section
        print("빠른 정답 찾기 시작")
        #doc.add_paragraph("\n\n\n\n\n")  # Add space before the '정답 및 해설' section
        paragraph = doc.add_paragraph("빠른 정답 찾기", style='Heading 1')
        paragraph.alignment = WD_PARAGRAPH_ALIGNMENT.CENTER

        # Access the first run of the paragraph to set the font size
        if paragraph.runs:
            paragraph.runs[0].font.size = Pt(20)
        else:
            # If there are no runs, add a new run and set its properties
            run = paragraph.add_run("빠른 정답 찾기")
            run.font.size = Pt(20)

        #doc.add_page_break()

        # Attempt to retrieve section titles
        try:
            section_titles = list(sections_dict.keys())  # Get all the section titles
            #self.app_logger.info("Section Titles: %s", section_titles)
        except IndexError:
            # Handle the error or pass if there's nothing specific to do
            messagebox.showerror("오류", "●추출할 지문이 없습니다. '추출할 지문번호 입력' 창에 지문번호가 올바르게 입력되어 있는지 확인하세요.")
            self.app_logger.info("●[오류] 추출할 지문이 없습니다.")
            return
        except Exception as e:
            # It's good practice to catch other potential errors
            self.app_logger.info(f"An unexpected error occurred: {e}")
            return

        # Determine the last section title, excluding specific keywords
        try:
            filtered_section_titles = [
                title for title in section_titles 
                if "동반의어_Normal" not in title and 
                "영영정의_Normal" not in title and 
                "커스텀" not in title and 
                "요약문" not in title and title != "동반의어" and title != "영영정의"
            ]
            if filtered_section_titles:
                last_section_title = filtered_section_titles[-1]  # Get the last section title
            else:
                last_section_title = section_titles[-1]  # Fallback to the last section title
        except IndexError:
            # Handle the error or notify the user
            def nothing_to_export():
                error_dialog = Toplevel()
                error_dialog.title("오류")
                error_dialog.geometry("500x150")
                error_dialog.grab_set()  # Make the dialog modal
                error_dialog.focus_set()  # Make the dialog modal
                error_dialog.grid_rowconfigure(0, weight=1)
                error_dialog.grid_columnconfigure(0, weight=1)

                CTkLabel(
                    error_dialog, 
                    fg_color="white", 
                    width=300, 
                    height=120, 
                    text="-추출할 지문이 없습니다.\n'추출할 지문번호 입력'창에 추출할 지문번호가 올바르게 입력되어 있는지 확인하세요.", 
                    text_color="black"
                ).grid(row=0, column=0, sticky="nsew")

                # Frame for buttons
                button_frame = CTkFrame(error_dialog, fg_color="white", width=300, height=30, corner_radius=0)
                button_frame.grid(row=1, column=0, sticky="swe")
                button_frame.grid_columnconfigure(0, weight=1)

                # "OK" button action
                def on_ok():
                    self.close_export_popup = True
                    error_dialog.grab_release()
                    error_dialog.destroy()

                # Create "OK" button
                ok_button = CTkButton(
                    button_frame, 
                    border_width=2, 
                    border_color="gray", 
                    text="OK", 
                    text_color="black", 
                    width=70, 
                    height=30, 
                    command=on_ok, 
                    fg_color="#FEF9E0", 
                    hover_color="#DDA15C"
                )
                ok_button.grid(row=0, column=0, padx=10, pady=10)

            self.close_export_popup = False
            nothing_to_export()
            return
        except Exception as e:
            # It's good practice to catch other potential errors
            self.app_logger.info(f"An unexpected error occurred: {e}")
            return

        # Handle sections_dict for answers and explanations
        self.빠른정답찾기용(doc, sections_dict, last_section_title)



















        # Save the Word document
        try:
            doc.save(output_doc)


        except Exception as e:
            self.app_logger.error(f"Failed to save Word document '{output_doc}': {e}")
            return

        # Merge with cover if option is set
        if self.applyoption_var.get() == "1":
            try:
                self.merge_docs(output_doc)
            except Exception as e:
                self.app_logger.error(f"Failed to merge documents: {e}")


        output_doc = unicodedata.normalize('NFC', output_doc)
        self.app_logger.info(f"-------------------\n편집본 추출 완료\n-------------------\n편집본 경로: {output_doc}")
        self.close_export_popup = True





    def determine_function_name(self, sheet_type):
        """
        Determines the appropriate editing function name based on the sheet_type.

        Parameters:
            sheet_type (str): The type of sheet being processed.

        Returns:
            str: The corresponding editing function name.
        """
        # Special case for '내용일치(한)'
        if "내용일치(한)" in sheet_type:
            return "내용일치한_편집"

        """
        # Check if it's a mock test sheet by looking for '_실모' suffix or specific identifiers
        if "실전모의고사" in sheet_type or "_실모" in sheet_type:
            # Extract the base type without '_실모' if present
            base_type = sheet_type.replace("_실모", "").strip()
            # Construct the function name by appending '_편집_실모'
            function_name = f"{base_type}_편집_실모"
            print(f"Determined mock test function name: {function_name}")  # Debug statement
            return function_name
        """

        function_name = sheet_type.replace('_Normal', '_편집').replace('_Hard', '_편집')
        

        print(f"Determined function name: {function_name}")  # Debug statement

        return function_name




    def process_question_part(self, doc, function_name, title, passage, result, answer_part, explanation_part):
        #print("process_question_part called")
        #print(f"Processing with function: {function_name}")  # Debug

        func = FUNCTION_MAP.get(function_name)
        if not func:
            self.app_logger.error(f"Function {function_name} not found in FUNCTION_MAP.")
            # Return default values to prevent further processing
            return title, passage, "", "", answer_part, explanation_part, None, None

        sig = inspect.signature(func)
        params = sig.parameters

        # Prepare arguments based on the function's parameters
        try:
            if 'doc' in params:
                # Function requires 'doc'
                processed = func(title, passage, result, answer_part, explanation_part, doc, self.app_logger)
            else:
                # Function does not require 'doc'
                processed = func(title, passage, result, answer_part, explanation_part, self.app_logger)
        except Exception as e:
            self.app_logger.error(f"Error executing {function_name}: {e}")
            # Return default values in case of error
            return title, passage, "", "", answer_part, explanation_part, None, None

        # Initialize default return values
        question_line = ""
        question_part = ""
        cell = None
        table = None



        # Determine the structure of the returned tuple
        if processed and isinstance(processed, tuple):
            #if "연결어" in function_name:
            #    title, passage, question_part, answer_part, explanation_part, doc, cell, table = processed[:8]

            if len(processed) >= 6:
                # Expecting at least: title, passage, question_line, question_part, answer_part, explanation_part
                title, passage, question_line, question_part, answer_part, explanation_part = processed[:6]
            elif len(processed) >= 5:
                # Some functions might not return question_line
                title, passage, question_part, answer_part, explanation_part = processed[:5]
                question_line = ""
            else:
                self.app_logger.error(f"Unexpected return format from {function_name}.")
                return title, passage, "", "", answer_part, explanation_part, None, None

            # Check if cell and table are returned
            #if len(processed) >= 8:
            #    # Functions that return cell and table as well
            #    cell, table = processed[6], processed[7]

        else:
            self.app_logger.error(f"Unexpected return format from {function_name}.")
            return title, passage, "", "", answer_part, explanation_part, None, None

        return title, passage, question_line, question_part, answer_part, explanation_part, cell, table




    def show_error_and_log(self, message):
        messagebox.showerror("오류", message)
        self.app_logger.error(message)

    def add_answer_and_explanation(self, doc):
        self.app_logger.info("해설지 시작")
        doc.add_paragraph("\n\n\n\n\n")
        paragraph = doc.add_paragraph("정답 및 해설", style='Heading 1')
        paragraph.alignment = WD_PARAGRAPH_ALIGNMENT.CENTER

        if paragraph.runs:
            paragraph.runs[0].font.size = Pt(20)
        else:
            run = paragraph.add_run("정답 및 해설")
            run.font.size = Pt(20)

        doc.add_page_break()





    def handle_regular_section_해설지(self, doc, section_title, title_answer_explanations, column_configurations, sections_dict):
        #print(f"handle_regular_section_해설지 called")

        if "동반의어_Normal" in section_title or "영영정의_Normal" in section_title or "커스텀_Normal" in section_title or "요약문_Normal" in section_title:
            return
        if section_title == "동반의어" or section_title == "영영정의" or section_title == "커스텀" or section_title == "요약문":
            return

        answer_section_title = f"{section_title}_정답"
        doc.add_paragraph(answer_section_title, style='Heading_ans')

        빈칸암기flag = False
        cleaned_section_title = re.sub(r'^\d+\.\s+', '', section_title)

        if "빈칸암기" in cleaned_section_title:
            빈칸암기flag = True
        else:
            base_section_title = cleaned_section_title.split('_')[0]

        #print(f"Base section title: {base_section_title}")  # Debug statement


        if not 빈칸암기flag:
            
            #print(f"base_section_title: {base_section_title}")

            if base_section_title in column_configurations:
                col_count, headers, widths = column_configurations[base_section_title]
                #print(f"Configuration found: {base_section_title} -> {col_count}, {headers}, {widths}")  # Debug

                table = doc.add_table(rows=1, cols=col_count)

                # Set header row
                for idx, header in enumerate(headers):
                    table.cell(0, idx).text = header

                # Apply styles to header
                for cell in table.rows[0].cells:
                    self.set_cell_font_hdr(cell, self.selected_font_for_exp, self.size_exp)
                    self.set_cell_border(
                        cell,
                        top={"sz": 8, "val": "single", "color": "auto"},
                        bottom={"sz": 8, "val": "single", "color": "auto"},
                        left={"sz": 8, "val": "single", "color": "auto"},
                        right={"sz": 8, "val": "single", "color": "auto"}
                    )
                    self.set_cell_margins(cell, top=1, bottom=1, left=1, right=1)

                # Add data rows
                for title, answer_part, explanation_part in title_answer_explanations:
                    cells = table.add_row().cells

                    if "영작" in base_section_title:
                        cells[0].text = title
                        #print(f"explanation_part: {explanation_part}")
                        if explanation_part == "" or explanation_part == None:
                            self.app_logger.error(f"{base_section_title} - {title}: 해설 부분이 비어 있습니다. 해당 문제를 재출제하세요.")
                            messagebox.showerror("Error", f"{base_section_title} - {title}: 해설 부분이 비어 있습니다. 해당 문제를 재출제하세요.")
                            continue
                        cells[1].text = explanation_part

                    else:
                        cells[0].text = title
                        cells[1].text = answer_part
                        if col_count == 3:
                            if (
                                not any(k in cleaned_section_title for k in ["어휘1", "어법1", "어휘2", "순서", "삽입", "빈칸암기"]) 
                                and not explanation_part
                            ):
                                self.show_error_and_log(
                                    f"오류: '{cleaned_section_title}' 유형의 '{title}' 지문에서 문제 오류가 발견되었습니다.\n"
                                    f"(오류내용: '해설'칸이 비어 있음)\n해당 문제를 삭제하거나 재출제하세요.\n\n"
                                    f"(재출제 방법: 해당 유형, 해당 지문번호의 '문제' 칸을 더블클릭하여 내용을 지운 후 다시 'Make Questions' → '출제 시작'을 클릭하세요.)"
                                )
                                return
                            else:
                                cells[2].text = explanation_part if explanation_part else ""



                    # Apply styles to content cells
                    for cell in cells:
                        self.set_cell_paragraph_style(cell, 'Normal_exp')
                        self.set_cell_border(
                            cell,
                            top={"sz": 8, "val": "single", "color": "auto"},
                            bottom={"sz": 8, "val": "single", "color": "auto"},
                            left={"sz": 8, "val": "single", "color": "auto"},
                            right={"sz": 8, "val": "single", "color": "auto"}
                        )
                        self.set_cell_margins(cell, top=5, bottom=5, left=5, right=5)

                    # Set column widths
                    #for idx, width in enumerate(widths):
                    #    table.rows[-1].cells[idx].width = width

                    # Set the widths for each column
                    for row in table.rows:
                        for idx, width in enumerate(widths):
                            row.cells[idx].width = width


            else:
                
                print(f"No column configuration found for section: {base_section_title}")  # Debug
                


        else: ###빈칸암기 편집용
            
            for title, answer_part, explanation_part in title_answer_explanations:
                para_title = doc.add_paragraph(style='Heading_num')
                para_title.add_run(str(title))

                # Use FUNCTION_MAP to retrieve the function
                function_name = '빈칸암기_정답_편집'  # Ensure this key exists in FUNCTION_MAP

                # Call the function using process_question_part
                self.process_question_part(
                    doc, function_name, title, answer_part, answer_part, answer_part, explanation_part
                )





    def handle_mock_test_문제지(self, wb, row_key, doc, section_number, sections_dict):
        """
        Handles the extraction and formatting of mock test sheets.

        Parameters:
            wb (Workbook): The loaded Excel workbook.
            row_key (str): The key identifying the mock test sheet. (실전모의 몇 회인지 표시)
            doc (Document): The Word document being generated.
            section_number (int): The current section number.
            sections_dict (defaultdict): Dictionary to store section data.
        """
        if row_key not in wb.sheetnames:
            self.app_logger.error(f"Sheet {row_key} does not exist in the workbook.")
            return

        extracting_sheet = wb[row_key]
        section_title = f"{section_number}. {row_key}"
        doc.add_paragraph(section_title, style='Heading 1')

        passage_id_to_row_data = {}
        for row in extracting_sheet.iter_rows(min_row=1, values_only=True):
            if row[0]:
                passage_id = row[0].strip()
                passage_id_to_row_data[passage_id] = row[:5]  # Assuming first 5 columns

        question_number = 1

        #print("passage_id_to_row_data\n")
        #print(passage_id_to_row_data)


        for passage_id, row_data in passage_id_to_row_data.items():
            # Ensure row_data has at least 5 elements
            row_data = tuple("" if cell is None else cell for cell in row_data)
            if len(row_data) < 5:
                row_data += ("",) * (5 - len(row_data))
            passageid, passage, result, answer_part, explanation_part = row_data[:5]

            if not passageid:
                continue

            try:
                유형 = passageid.split(" // ")[1].strip()
                원래title = passageid.split(" // ")[0].strip()
            except IndexError:
                self.app_logger.error(f"Invalid passage_id format: {passageid}")
                continue

            title = f"{question_number}."
            해설지로보낼title = f"{title} {원래title}"

            if result in ["문제 미출제 (문장 수 부족)", "", None] or (result and result.startswith("Error")):
                self.app_logger.info(f"{passageid}: 문제 조건에 맞지 않아 추출하지 않습니다.")
                continue

            유형 = 유형 + "_실모"
            #print(f"유형: {유형}")

            function_name = 유형.replace('_Normal', '_편집').replace('_Hard', '_편집').replace("내용일치(한)", "내용일치한")
            #function_name = self.determine_function_name(유형)

            if function_name not in FUNCTION_MAP:
                self.app_logger.error(f"Function {function_name} not found in FUNCTION_MAP.")
                continue

            # Process the question part
            processed = self.process_question_part(
                doc, function_name, title, passage, result, answer_part, explanation_part
            )

            if processed:
                title, passage, question_line, question_part, answer_part, explanation_part, cell, table = processed

                title = title + " " + question_line
                title = title.strip()
                # Create a paragraph for the title
                para_title = doc.add_paragraph(style='Normal_left')



                if "함축의미" in 유형:
                    self.테이블없는본문_괄호에볼드밑줄더하기(para_title, title)

                elif "영작2" in 유형:
                    self.테이블없는본문_영작단어수에볼드밑줄더하기(para_title, title)
                
                else:
                    # Determine which words need to be formatted
                    words_to_format = []
                    if '없는' in title:
                        words_to_format.append('없는')
                    if '않는' in title:
                        words_to_format.append('않는')
                    if '않은' in title:
                        words_to_format.append('않은')
                    if '틀린' in title:
                        words_to_format.append('틀린')
                    if '2개' in title:
                        words_to_format.append('2개')
                    if '3개' in title:
                        words_to_format.append('3개')


                    # Add the title with formatting
                    self.add_formatted_text(para_title, title, words_to_format)

                


                #지문
                # Create the table for the passage
                
                if "영작1" in 유형:

                    # Find and capture the "문제파트"
                    문제파트 = ""
                    # Use regex to match the entire problem part before the actual content
                    match = re.search(r"다음의 밑줄 친.*?\)", question_part)
                    if match:
                        문제파트 = match.group(0)  # Capture the entire "문제파트"

                    # Remove the "문제파트" from the original text
                    question_part = question_part.replace(문제파트, "").strip()

                    # Now safely split the remaining text at the first "[보기]" to get 지문파트 and 보기파트
                    if '[보기]' in question_part:
                        지문파트, 보기파트 = question_part.split('[보기]', 1)  # Use 1 to ensure only the first occurrence is used
                        지문파트 = 지문파트.strip()  # Clean up any surrounding whitespace
                        보기파트 = '[보기] ' + 보기파트.strip()  # Add [보기] back and clean whitespace
                    else:
                        # Handle cases where "[보기]" might not be present
                        지문파트 = question_part.strip()
                        보기파트 = ""                                            

                    # 지문파트
                    table = doc.add_table(rows=1, cols=1)
                    cell = table.cell(0, 0)
                    cell.text = ""
                    paragraphs = 지문파트.split('\n')
                    add_bold_underline_to_brackets(cell, paragraphs)  # Add bold + underline to brackets


                    # 보기파트
                    보기파트 = 보기파트 + "\n\n → ____________________________________________________________________________________________"

                    for paragraph in 보기파트.split('\n'):
                        doc.add_paragraph(paragraph, style='Normal_modified')
                    


                elif "영작2" in 유형:

                    # Find and capture the "문제파트"
                    문제파트 = ""
                    # Use regex to match the entire problem part before the actual content
                    match = re.search(r"\[보기\]에.*?\)", question_part)
                    if match:
                        문제파트 = match.group(0)  # Capture the entire "문제파트"

                    # Remove the "문제파트" from the original text
                    question_part = question_part.replace(문제파트, "").strip()

                    # Now safely split the remaining text at the first "[보기]" to get 지문파트 and 보기파트
                    if '[보기]' in question_part:
                        지문파트, 보기파트 = question_part.split('[보기]', 1)  # Use 1 to ensure only the first occurrence is used
                        지문파트 = 지문파트.strip()  # Clean up any surrounding whitespace
                        보기파트 = '[보기] ' + 보기파트.strip()  # Add [보기] back and clean whitespace
                    else:
                        # Handle cases where "[보기]" might not be present
                        지문파트 = question_part.strip()
                        보기파트 = ""                                            



                    # 지문파트
                    table = doc.add_table(rows=1, cols=1)
                    cell = table.cell(0, 0)
                    cell.text = ""
                    paragraphs = 지문파트.split('\n')
                    add_bold_underline_to_brackets(cell, paragraphs)  # Add bold + underline to brackets

                    # 보기파트
                    보기파트 = 보기파트 + "\n\n → ____________________________________________________________________________________________"

                    for paragraph in 보기파트.split('\n'):
                        doc.add_paragraph(paragraph, style='Normal_modified')




                elif "요약" in 유형:

                    question_part = question_part.replace("윗글의 내용을 한 문장으로 요약하고자 한다. 빈칸 (A), (B)에 들어갈 말로 가장 적절한 것은?\n", "")

                    #문제 라인 입력
                    #question_line = "다음 글의 내용을 한 문장으로 요약하고자 한다. 빈칸 (A), (B)에 들어갈 말로 가장 적절한 것은?"
                    #para_question_line = doc.add_paragraph(question_line, style='Normal_modified')
                    #para_question_line.alignment = WD_ALIGN_PARAGRAPH.LEFT


                    #지문 테이블에 입력
                    table = doc.add_table(rows=1, cols=1)
                    cell = table.cell(0, 0)
                    cell.text = ""

                    paragraphs = passage.split('\n')
                    add_bold_underline_to_brackets(cell, paragraphs)

                    #요약문 기호 입력
                    요약문기호 = "⇩"
                    para = doc.add_paragraph(요약문기호, style='Normal_modified')
                    para.alignment = WD_ALIGN_PARAGRAPH.CENTER


                    #문제를 요약문과 선지로 쪼개기
                    split_point = question_part.find("(A) -")
                    요약문파트 = question_part[:split_point].strip()
                    선지파트 = question_part[split_point:].strip()

                    #print(f"요약문파트: {요약문파트}")
                    #print(f"선지파트: {선지파트}")

                    #요약문 파트 괄호 제거하고 표에 넣기
                    table2 = doc.add_table(rows=1, cols=1)
                    cell2 = table2.cell(0, 0)
                    cell2.text = ""

                    요약문파트_paragraphs = 요약문파트.split('\n')
                    remove_brackets(cell2, 요약문파트_paragraphs)
                    

                    # 선지파트 탭 적용 후 넣기
                    선지파트 = 선지파트.replace("(A) - (B)", "          (A)\t\t       (B)")
                    선지파트 = 선지파트.replace(" -", "\t...\t")

                    for paragraph_text in 선지파트.split('\n'):
                        paragraph = doc.add_paragraph(paragraph_text, style='Normal_modified')  # Now paragraph is a Paragraph object

                        # Set custom tab stops at desired intervals for the options paragraph
                        tab_stops = paragraph.paragraph_format.tab_stops
                        for tab_pos in range(1, 2):  # Assuming we want a tab stop between each option
                            tab_stops.add_tab_stop(Inches(1.1 * tab_pos))

                
                    self.set_cell_border(cell2, top={"sz": 8, "val": "single", "color": "auto"},
                                    bottom={"sz": 8, "val": "single", "color": "auto"},
                                    left={"sz": 8, "val": "single", "color": "auto"},
                                    right={"sz": 8, "val": "single", "color": "auto"})
                
                    self.set_table_cell_margins(table2, top=5, bottom=5, left=7, right=7)



                else:
                    table = doc.add_table(rows=1, cols=1)
                    cell = table.cell(0, 0)
                    cell.text = ""
                    paragraphs = passage.split('\n')
                    add_bold_underline_to_brackets(cell, paragraphs)

                    #문제
                    for paragraph in question_part.split('\n'):
                        doc.add_paragraph(paragraph, style='Normal_modified')                                        





                # Apply styles
                if cell is not None:
                    self.set_cell_border(
                        cell,
                        top={"sz": 8, "val": "single", "color": "auto"},
                        bottom={"sz": 8, "val": "single", "color": "auto"},
                        left={"sz": 8, "val": "single", "color": "auto"},
                        right={"sz": 8, "val": "single", "color": "auto"}
                    )
                    if table is not None:
                        self.set_table_cell_margins(table, top=5, bottom=5, left=7, right=7)
                doc.add_paragraph(style='Normal_modified')




                # Check if answer_part is empty
                if not answer_part and "영작" not in 유형:
                    self.show_error_and_log(
                        f"오류: '{row_key}' 유형의 '{title}' 지문에서 문제 오류가 발견되었습니다.\n"
                        f"(오류내용: '정답'칸이 비어 있음)\n해당 문제를 삭제하거나 재출제하세요."
                    )
                    return

                sections_dict[section_title].append((해설지로보낼title, answer_part, explanation_part))
            question_number += 1



    def handle_mock_test_section_해설지(self, doc, section_title, title_answer_explanations, column_configurations):
        """
        Handles the formatting and insertion of mock test sections into the Word document.

        Parameters:
            doc (Document): The Word document being generated.
            section_title (str): The title of the mock test section.
            title_answer_explanations (list): List of tuples containing title, answer_part, and explanation_part.
            column_configurations (dict): Dictionary containing column configurations.
        """
        answer_section_title = f"{section_title}_정답"
        doc.add_paragraph(answer_section_title, style='Heading_ans')

        # Fetch column configurations for mock tests
        if '실전모의고사' not in column_configurations:
            self.app_logger.error(f"No column configuration found for '실전모의고사'.")
            return

        col_count, headers, widths = column_configurations['실전모의고사']
        table = doc.add_table(rows=1, cols=col_count)

        # Set header row
        for idx, header in enumerate(headers):
            table.cell(0, idx).text = header

        # Apply styles to header
        for cell in table.rows[0].cells:
            self.set_cell_font_hdr(cell, self.selected_font_for_exp, self.size_exp)
            self.set_cell_border(
                cell,
                top={"sz": 8, "val": "single", "color": "auto"},
                bottom={"sz": 8, "val": "single", "color": "auto"},
                left={"sz": 8, "val": "single", "color": "auto"},
                right={"sz": 8, "val": "single", "color": "auto"}
            )
            self.set_cell_margins(cell, top=1, bottom=1, left=1, right=1)

        # Add data rows
        for title, answer_part, explanation_part in title_answer_explanations:
            cells = table.add_row().cells
            cells[0].text = str(title)
            cells[1].text = answer_part
            cells[2].text = explanation_part if explanation_part else ""

            # Apply styles to content cells
            for cell in cells:
                self.set_cell_paragraph_style(cell, 'Normal_exp')
                self.set_cell_border(
                    cell,
                    top={"sz": 8, "val": "single", "color": "auto"},
                    bottom={"sz": 8, "val": "single", "color": "auto"},
                    left={"sz": 8, "val": "single", "color": "auto"},
                    right={"sz": 8, "val": "single", "color": "auto"}
                )
                self.set_cell_margins(cell, top=5, bottom=5, left=5, right=5)

            # Set column widths
            #for idx, width in enumerate(widths):
            #    table.rows[-1].cells[idx].width = width
            # Set the widths for each column
            for row in table.rows:
                for idx, width in enumerate(widths):
                    row.cells[idx].width = width





    def split_multi_questions(self, text):
        """
        Splits text by '문제1)', '문제2)', etc.
        Returns a list of (question_number, question_text).
        E.g.:
            문제1) 윗글의 제목으로 가장 적절한 것은?...
            문제2) 밑줄 친 (a)~(e) 중에서...
        becomes:
            [
            ("문제1)", "윗글의 제목으로 가장 적절한 것은?..."),
            ("문제2)", "밑줄 친 (a)~(e) 중에서...")
            ]
        """
        # This pattern captures '문제1)' or '문제2)' or up to 문제9) etc.
        # If you expect more digits, you can adjust the pattern
        pattern = r"(문제\d+\))"

        # Split while keeping the delimiter
        split_text = re.split(pattern, text)

        # e.g. split_text might look like ["", "문제1)", " ...question1.. ", "문제2)", " ...question2..." ]
        # so we pair them up
        results = []
        # first element might be empty. We'll pair them as (문제X), rest_of_text
        i = 1
        while i < len(split_text):
            question_number = split_text[i].strip()
            question_text = split_text[i+1].strip() if i+1 < len(split_text) else ""
            results.append((question_number, question_text))
            i += 2

        return results


    def split_answers(self, answer_part):
        """
        Splits two answers, e.g. "①④" -> ["①", "④"].
        This is purely an example; if you have
        more answers or a different format, adjust accordingly.
        """
        # If answers are always two circled numbers stuck together,
        # you can just split them individually.
        # e.g. "①④" -> ["①", "④"]
        # If you might have ①④⑤, adapt your logic accordingly.
        return list(answer_part)


    def split_explanations(self, explanation_part):
        """
        Splits explanations by '문제1)', '문제2)', etc.
        Returns a dict: { "문제1)": "explanation1...", "문제2)": "explanation2..." }
        or a list of tuples like the questions.
        """
        # Reuse the same logic from split_multi_questions, if you want consistent parsing:
        items = self.split_multi_questions(explanation_part)
        # items looks like [("문제1)", "..."), ("문제2)", "...")]
        # Turn it into a dict for convenient access.
        exp_dict = {k: v for (k, v) in items}
        return exp_dict



    def handle_past_test_문제지(self, wb, row_key, doc, section_number, sections_dict):
        """
        Handles the extraction and formatting of mock test sheets.

        Parameters:
            wb (Workbook): The loaded Excel workbook.
            row_key (str): The key identifying the mock test sheet. (실전모의 몇 회인지 표시)
            doc (Document): The Word document being generated.
            section_number (int): The current section number.
            sections_dict (defaultdict): Dictionary to store section data.
        """
        if row_key not in wb.sheetnames:
            self.app_logger.error(f"Sheet {row_key} does not exist in the workbook.")
            return

        extracting_sheet = wb[row_key]
        section_title = f"{section_number}. {row_key}"
        doc.add_paragraph(section_title, style='Heading 1')

        passage_id_to_row_data = {}
        for row in extracting_sheet.iter_rows(min_row=1, values_only=True):
            if row[0]:
                passage_id = row[0].strip()
                passage_id_to_row_data[passage_id] = row[:5]  # Assuming first 5 columns

        question_number = 1

        #print("passage_id_to_row_data\n")
        #print(passage_id_to_row_data)


        for passage_id, row_data in passage_id_to_row_data.items():
            # Ensure row_data has at least 5 elements
            row_data = tuple("" if cell is None else cell for cell in row_data)
            if len(row_data) < 5:
                row_data += ("",) * (5 - len(row_data))
            passageid, passage, result, answer_part, explanation_part = row_data[:5]

            if not passageid:
                continue

            try:
                유형 = passageid.split(" // ")[1].strip()
                원래title = passageid.split(" // ")[0].strip()
            except IndexError:
                self.app_logger.error(f"Invalid passage_id format: {passageid}")
                continue

            title = f"{question_number}."
            #해설지로보낼title = f"{title} {원래title}"
            
            해설지로보낼title = f"{question_number}"

            if result in ["문제 미출제 (문장 수 부족)", "", None] or (result and result.startswith("Error")):
                self.app_logger.info(f"{passageid}: 문제 조건에 맞지 않아 추출하지 않습니다.")
                continue

            유형 = 유형 + "_기출"
            #print(f"유형: {유형}")

            function_name = 유형.replace('_Normal', '_편집').replace('_Hard', '_편집').replace("내용일치(한)", "내용일치한")
            #function_name = self.determine_function_name(유형)

            if function_name not in FUNCTION_MAP:
                self.app_logger.error(f"Function {function_name} not found in FUNCTION_MAP.")
                continue


            # Process the question part
            processed = self.process_question_part(
                doc, function_name, title, passage, result, answer_part, explanation_part
            )

            if processed:
                title, passage, question_line, question_part, answer_part, explanation_part, cell, table = processed



                # -----------------------------------------------------
                #  Check for "장문독해1"
                # -----------------------------------------------------
                if "장문독해1" in 유형:
                    """
                    1) We want each sub-question to have its own question_number
                    (19, 20, etc. instead of 1, 2).
                    2) We'll display the `question_line` first (or not, depending on your design).
                    3) We'll display the passage once in a table.
                    4) Then for each sub-question (문제1), (문제2), we increment question_number.
                    """
                    
                    question_text = "[" + str(question_number) + "~" + str(question_number + 1) + "] " + question_line
                    # Create a new paragraph (with your custom style if you wish)
                    paragraph = doc.add_paragraph('', style='Normal_modified')
                    # Add a run to this paragraph
                    run = paragraph.add_run(question_text)
                    # Make the run bold
                    run.bold = True


                    # (★) -- Put the passage in a table once
                    table = doc.add_table(rows=1, cols=1)
                    cell = table.cell(0, 0)
                    cell.text = ""
                    paragraphs = passage.split('\n')
                    add_bold_underline_to_brackets(cell, paragraphs)
                    self.set_cell_border(
                        cell,
                        top={"sz": 8, "val": "single", "color": "auto"},
                        bottom={"sz": 8, "val": "single", "color": "auto"},
                        left={"sz": 8, "val": "single", "color": "auto"},
                        right={"sz": 8, "val": "single", "color": "auto"}
                    )
                    self.set_table_cell_margins(table, top=5, bottom=5, left=7, right=7)

                    # 3) Split the multi-question text
                    question_list = self.split_multi_questions(result)
                    # e.g. [("문제1)", "..."), ("문제2)", "...")]

                    # 4) Split answers and explanations
                    answers = self.split_answers(answer_part)  # e.g. ["①","④"]
                    explanations_dict = self.split_explanations(explanation_part)
                    
                    # (★) -- Instead of sub_q_number = 1, start from current question_number
                    sub_q_number = question_number  

                    # 5) For each sub-question...
                    for idx, (mq_key, mq_text) in enumerate(question_list):
                        # Typically "mq_key" is "문제1)", and "mq_text" is the content after that.
                        
                        # We want to do something like:
                        # 19. 윗글의 제목으로 가장 적절한 것은?
                        #  (choices)
                        # 20. 밑줄 친 (a)~(e)...
                        
                        # The code below: figure out the first line vs. the rest
                        lines = mq_text.split('\n', 1)
                        if len(lines) > 1:
                            first_line = lines[0].strip()
                            rest = lines[1]
                        else:
                            first_line = lines[0].strip()
                            rest = ""


                        # (★) -- Now produce the question line: e.g. "19. 윗글의 제목으로..."
                        question_label = f"{sub_q_number}. {first_line}"
                        #doc.add_paragraph(question_label, style='Normal_modified')

                        문제라인 = doc.add_paragraph(style='Normal_left')

                        words_to_format = []
                        if '없는' in question_label:
                            words_to_format.append('없는')
                        if '않는' in question_label:
                            words_to_format.append('않는')
                        if '않은' in question_label:
                            words_to_format.append('않은')
                        if '다른' in question_label:
                            words_to_format.append('다른')
                        if '틀린' in question_label:
                            words_to_format.append('틀린')                    
                        if '2개' in question_label:
                            words_to_format.append('2개')
                        if '3개' in question_label:
                            words_to_format.append('3개')

                        # Add the title with formatting
                        self.add_formatted_text(문제라인, question_label, words_to_format)

                        if rest:
                            for line in rest.split('\n'):
                                doc.add_paragraph(line, style='Normal_modified')

                        # After writing the sub-question...
                        if idx < len(question_list) - 1:
                            # Only add a blank line if this is NOT the last sub-question
                            doc.add_paragraph("")

                        # pick the matching answer and explanation
                        try:
                            answer_for_this = answers[sub_q_number - question_number]  
                            # Explanation: If sub_q_number == question_number, index=0
                            # If sub_q_number == question_number+1, index=1, etc.
                        except IndexError:
                            answer_for_this = "N/A"

                        explanation_for_this = explanations_dict.get(mq_key, "")
                        explanation_for_this = explanation_for_this.replace(mq_key, "").strip()




                        # Save in sections_dict so you can do 해설 later
                        if mq_key == "문제1)":
                            passageid해설지용 = passageid.replace("41~42", "41")
                        elif mq_key == "문제2)":
                            passageid해설지용 = passageid.replace("41~42", "42")
                        passageid해설지용 = passageid해설지용.split(" // ")[0].strip()


                        #sub_title_for_해설 = f"{sub_q_number}. {passageid해설지용}"
                        sub_title_for_해설 = f"{sub_q_number}"

                        sections_dict[section_title].append(
                            (sub_title_for_해설, answer_for_this, explanation_for_this)
                        )

                        # (★) -- increment question_number for the next sub-question
                        sub_q_number += 1

                    # (★) -- After finishing ALL sub-questions for this 장문독해1,
                    #         update our main question_number for the next top-level question
                    question_number = sub_q_number

                    # Add an extra paragraph if you like for spacing
                    doc.add_paragraph(style='Normal_modified')


                elif "장문독해2" in 유형:
                    """
                    장문독해2 block (3 sub-questions).
                    Essentially the same loop, but we *expect* 3 questions:
                    문젯글에 '문제1)', '문제2)', '문제3)' 
                    """
                    #doc.add_paragraph(question_line, style='Normal_modified')
                    question_text = "[" + str(question_number) + "~" + str(question_number + 2) + "] " + question_line
                    # Create a new paragraph (with your custom style if you wish)
                    paragraph = doc.add_paragraph('', style='Normal_modified')
                    # Add a run to this paragraph
                    run = paragraph.add_run(question_text)
                    # Make the run bold
                    run.bold = True





                    for paragraph in passage.split('\n'):
                        if "(A)" in paragraph or "(B)" in paragraph or "(C)" in paragraph or "(D)" in paragraph:                        
                            para = doc.add_paragraph(paragraph, style='Normal_modified')
                            para.alignment = WD_ALIGN_PARAGRAPH.CENTER


                        elif paragraph.startswith('*'):
                            passage_style = 'Normal_right'
                            doc.add_paragraph(paragraph, style=passage_style)

                        else:
                            table = doc.add_table(rows=1, cols=1)
                            cell = table.cell(0, 0)
                            cell.text = ""
                            paragraphs = paragraph.split('\n')
                            add_bold_underline_to_brackets(cell, paragraphs)
                        
                            self.set_cell_border(
                                cell,
                                top={"sz": 8, "val": "single", "color": "auto"},
                                bottom={"sz": 8, "val": "single", "color": "auto"},
                                left={"sz": 8, "val": "single", "color": "auto"},
                                right={"sz": 8, "val": "single", "color": "auto"}
                            )
                            self.set_table_cell_margins(table, top=5, bottom=5, left=7, right=7)


                    question_list = self.split_multi_questions(result)
                    
                    # (Optional) Confirm we got exactly 3 sub-questions:
                    # if len(question_list) != 3:
                    #     self.app_logger.error(f"장문독해2 should have 3 questions, found {len(question_list)}.")
                    #     continue  # or handle how you like

                    answers = self.split_answers(answer_part)
                    explanations_dict = self.split_explanations(explanation_part)
                    sub_q_number = question_number

                    for idx, (mq_key, mq_text) in enumerate(question_list):
                        lines = mq_text.split('\n', 1)
                        if len(lines) > 1:
                            first_line = lines[0].strip()
                            rest = lines[1]
                        else:
                            first_line = lines[0].strip()
                            rest = ""

                        question_label = f"{sub_q_number}. {first_line}"
                        #doc.add_paragraph(question_label, style='Normal_modified')

                        문제라인 = doc.add_paragraph(style='Normal_left')

                        words_to_format = []
                        if '없는' in question_label:
                            words_to_format.append('없는')
                        if '않는' in question_label:
                            words_to_format.append('않는')
                        if '않은' in question_label:
                            words_to_format.append('않은')
                        if '다른' in question_label:
                            words_to_format.append('다른')
                        if '틀린' in question_label:
                            words_to_format.append('틀린')                    
                        if '2개' in question_label:
                            words_to_format.append('2개')
                        if '3개' in question_label:
                            words_to_format.append('3개')

                        # Add the title with formatting
                        self.add_formatted_text(문제라인, question_label, words_to_format)

                        if rest:
                            for line in rest.split('\n'):
                                doc.add_paragraph(line, style='Normal_modified')
                        
                        # Insert a blank paragraph if we want spacing between each sub-question
                        if idx < len(question_list) - 1:
                            doc.add_paragraph("")
                        
                        try:
                            answer_for_this = answers[sub_q_number - question_number]
                        except IndexError:
                            answer_for_this = "N/A"

                        explanation_for_this = explanations_dict.get(mq_key, "")
                        explanation_for_this = explanation_for_this.replace(mq_key, "").strip()


                        # Save in sections_dict so you can do 해설 later
                        if mq_key == "문제1)":
                            passageid해설지용 = passageid.replace("43~45", "43")
                        elif mq_key == "문제2)":
                            passageid해설지용 = passageid.replace("43~45", "44")
                        elif mq_key == "문제3)":
                            passageid해설지용 = passageid.replace("43~45", "45")

                        passageid해설지용 = passageid해설지용.split(" // ")[0].strip()                        
                        #sub_title_for_해설 = f"{sub_q_number}. {passageid해설지용}"
                        sub_title_for_해설 = f"{sub_q_number}"

                        sections_dict[section_title].append(
                            (sub_title_for_해설, answer_for_this, explanation_for_this)
                        )

                        sub_q_number += 1

                    question_number = sub_q_number
                    doc.add_paragraph(style='Normal_modified')








                else:
                    # -----------------------------------------------------
                    # Default processing for normal cases (no "장문독해1" or "장문독해2")
                    # -----------------------------------------------------


                    title = title + " " + question_line
                    title = title.strip()
                    # Create a paragraph for the title
                    para_title = doc.add_paragraph(style='Normal_left')



                    if "함축의미" in 유형:
                        self.테이블없는본문_괄호에볼드밑줄더하기(para_title, title)

                    elif "영작2" in 유형:
                        self.테이블없는본문_영작단어수에볼드밑줄더하기(para_title, title)
                    
                    else:
                        # Determine which words need to be formatted
                        words_to_format = []
                        if '없는' in title:
                            words_to_format.append('없는')
                        if '않는' in title:
                            words_to_format.append('않는')
                        if '않은' in title:
                            words_to_format.append('않은')
                        if '틀린' in title:
                            words_to_format.append('틀린')                    
                        if '2개' in title:
                            words_to_format.append('2개')
                        if '3개' in title:
                            words_to_format.append('3개')


                        # Add the title with formatting
                        self.add_formatted_text(para_title, title, words_to_format)

                    


                    if "요약" in 유형:

                        question_part = question_part.replace("윗글의 내용을 한 문장으로 요약하고자 한다. 빈칸 (A), (B)에 들어갈 말로 가장 적절한 것은?\n", "")

                        #문제 라인 입력
                        #question_line = "다음 글의 내용을 한 문장으로 요약하고자 한다. 빈칸 (A), (B)에 들어갈 말로 가장 적절한 것은?"
                        #para_question_line = doc.add_paragraph(question_line, style='Normal_modified')
                        #para_question_line.alignment = WD_ALIGN_PARAGRAPH.LEFT


                        #지문 테이블에 입력
                        table = doc.add_table(rows=1, cols=1)
                        cell = table.cell(0, 0)
                        cell.text = ""

                        paragraphs = passage.split('\n')
                        add_bold_underline_to_brackets(cell, paragraphs)

                        #요약문 기호 입력
                        요약문기호 = "⇩"
                        para = doc.add_paragraph(요약문기호, style='Normal_modified')
                        para.alignment = WD_ALIGN_PARAGRAPH.CENTER


                        #문제를 요약문과 선지로 쪼개기
                        split_point = question_part.find("(A) -")
                        요약문파트 = question_part[:split_point].strip()
                        선지파트 = question_part[split_point:].strip()

                        #print(f"요약문파트: {요약문파트}")
                        #print(f"선지파트: {선지파트}")

                        #요약문 파트 괄호 제거하고 표에 넣기
                        table2 = doc.add_table(rows=1, cols=1)
                        cell2 = table2.cell(0, 0)
                        cell2.text = ""

                        요약문파트_paragraphs = 요약문파트.split('\n')
                        remove_brackets(cell2, 요약문파트_paragraphs)
                        

                        # 선지파트 탭 적용 후 넣기
                        선지파트 = 선지파트.replace("(A) - (B)", "          (A)\t\t       (B)")
                        선지파트 = 선지파트.replace(" -", "\t...\t")

                        for paragraph_text in 선지파트.split('\n'):
                            paragraph = doc.add_paragraph(paragraph_text, style='Normal_modified')  # Now paragraph is a Paragraph object

                            # Set custom tab stops at desired intervals for the options paragraph
                            tab_stops = paragraph.paragraph_format.tab_stops
                            for tab_pos in range(1, 2):  # Assuming we want a tab stop between each option
                                tab_stops.add_tab_stop(Inches(1.1 * tab_pos))

                    
                        self.set_cell_border(cell2, top={"sz": 8, "val": "single", "color": "auto"},
                                        bottom={"sz": 8, "val": "single", "color": "auto"},
                                        left={"sz": 8, "val": "single", "color": "auto"},
                                        right={"sz": 8, "val": "single", "color": "auto"})
                    
                        self.set_table_cell_margins(table2, top=5, bottom=5, left=7, right=7)


                    elif "무관한문장" in 유형:
                        #박스에 넣을 지문 없는 경우
                        for paragraph in question_part.split('\n'):
                            if paragraph.startswith('*'):
                                passage_style = 'Normal_right'
                            else:
                                passage_style = 'Normal_modified'
                            doc.add_paragraph(paragraph, style=passage_style)



                    else:
                        #박스에 넣을 지문
                        table = doc.add_table(rows=1, cols=1)
                        cell = table.cell(0, 0)
                        cell.text = ""
                        paragraphs = passage.split('\n')
                        add_bold_underline_to_brackets(cell, paragraphs)

                        #나머지 
                        #for paragraph in question_part.split('\n'):
                        #    doc.add_paragraph(paragraph, style='Normal_modified')                                        
                        for paragraph in question_part.split('\n'):
                            if paragraph.startswith('*'):
                                passage_style = 'Normal_right'
                            else:
                                passage_style = 'Normal_modified'
                            doc.add_paragraph(paragraph, style=passage_style)




                    # Apply styles
                    if cell is not None:
                        self.set_cell_border(
                            cell,
                            top={"sz": 8, "val": "single", "color": "auto"},
                            bottom={"sz": 8, "val": "single", "color": "auto"},
                            left={"sz": 8, "val": "single", "color": "auto"},
                            right={"sz": 8, "val": "single", "color": "auto"}
                        )
                        if table is not None:
                            self.set_table_cell_margins(table, top=5, bottom=5, left=7, right=7)
                    doc.add_paragraph(style='Normal_modified')




                    # Check if answer_part is empty
                    if not answer_part and "영작" not in 유형:
                        self.show_error_and_log(
                            f"오류: '{row_key}' 유형의 '{title}' 지문에서 문제 오류가 발견되었습니다.\n"
                            f"(오류내용: '정답'칸이 비어 있음)\n해당 문제를 삭제하거나 재출제하세요."
                        )
                        return

                    sections_dict[section_title].append((해설지로보낼title, answer_part, explanation_part))

                    question_number += 1





    def handle_past_test_section_해설지(self, doc, section_title, title_answer_explanations, column_configurations):
        print("handle_past_test_section_해설지 called")
        """
        Handles the formatting and insertion of mock test sections into the Word document.

        Parameters:
            doc (Document): The Word document being generated.
            section_title (str): The title of the mock test section.
            title_answer_explanations (list): List of tuples containing title, answer_part, and explanation_part.
            column_configurations (dict): Dictionary containing column configurations.
        """
        answer_section_title = f"{section_title}_정답"
        doc.add_paragraph(answer_section_title, style='Heading_ans')

        # Fetch column configurations for mock tests
        if '기출문제세트' not in column_configurations:
            self.app_logger.error(f"No column configuration found for '기출문제세트'.")
            return

        col_count, headers, widths = column_configurations['기출문제세트']
        #print(f"col_count: {col_count}, headers: {headers}, widths: {widths}")
        table = doc.add_table(rows=1, cols=col_count)

        # Set header row
        for idx, header in enumerate(headers):
            table.cell(0, idx).text = header

        # Apply styles to header
        for cell in table.rows[0].cells:
            self.set_cell_font_hdr(cell, self.selected_font_for_exp, self.size_exp)
            self.set_cell_border(
                cell,
                top={"sz": 8, "val": "single", "color": "auto"},
                bottom={"sz": 8, "val": "single", "color": "auto"},
                left={"sz": 8, "val": "single", "color": "auto"},
                right={"sz": 8, "val": "single", "color": "auto"}
            )
            self.set_cell_margins(cell, top=1, bottom=1, left=1, right=1)

        # Add data rows
        for title, answer_part, explanation_part in title_answer_explanations:
            cells = table.add_row().cells
            cells[0].text = str(title)
            cells[1].text = answer_part
            cells[2].text = explanation_part if explanation_part else ""

            # Apply styles to content cells
            for cell in cells:
                self.set_cell_paragraph_style(cell, 'Normal_exp')
                self.set_cell_border(
                    cell,
                    top={"sz": 8, "val": "single", "color": "auto"},
                    bottom={"sz": 8, "val": "single", "color": "auto"},
                    left={"sz": 8, "val": "single", "color": "auto"},
                    right={"sz": 8, "val": "single", "color": "auto"}
                )
                self.set_cell_margins(cell, top=5, bottom=5, left=5, right=5)

            # Set column widths
            #for idx, width in enumerate(widths):
            #    table.rows[-1].cells[idx].width = width

            # Set the widths for each column
            for row in table.rows:
                for idx, width in enumerate(widths):
                    row.cells[idx].width = width




    def get_column_configurations(self):
        print("get_column_configurations called")

        return {
            '어휘1단계': (2, ('번호', '정답'), (Inches(1), Inches(2))),
            '어휘2단계': (2, ('번호', '정답'), (Inches(1), Inches(2))),
            '어휘3단계': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.3), Inches(5.7))),
            '내용일치': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.5), Inches(5.5))),
            '내용일치(한)': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.5), Inches(5.5))),
            '밑줄의미': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.3), Inches(5.7))),
            '함축의미': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.3), Inches(5.7))),
            '빈칸추론': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.3), Inches(5.7))),
            '추론': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.3), Inches(5.7))),
            '추론불가': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.3), Inches(5.7))),
            '순서': (2, ('번호', '정답'), (Inches(1), Inches(1))),
            '삽입': (2, ('번호', '정답'), (Inches(1), Inches(1))),
            '주제': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.3), Inches(5.7))),
            '제목': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.3), Inches(5.7))),
            '요지': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.3), Inches(5.7))),
            '연결어_Normal': (2, ('번호', '정답'), (Inches(1), Inches(2))),
            '연결어_Hard': (3, ('번호', '정답', '해설'), (Inches(1), Inches(2), Inches(2))),
            '연결어': (3, ('번호', '정답', '해설'), (Inches(1), Inches(2), Inches(2))),
            '요약': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.3), Inches(5.7))),
            '어법1단계': (2, ('번호', '정답'), (Inches(1), Inches(2))),
            '어법2단계': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.3), Inches(5.7))),
            '빈칸암기': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.3), Inches(5.7))),
            '어휘종합': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.7), Inches(5.3))),
            '내용종합': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.7), Inches(5.3))),
            '영작1': (2, ('번호', '정답'), (Inches(1), Inches(6))),
            '영작2': (2, ('번호', '정답'), (Inches(1), Inches(6))),
            '동형문제': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.3), Inches(5.7))),
            '동반의어문제1': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.5), Inches(1.5))),
            '동반의어문제2': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.5), Inches(1.5))),
            '영영정의문제': (3, ('번호', '정답', '해설'), (Inches(1), Inches(1.2), Inches(1.2))),
            '실전모의고사': (3, ('번호', '정답', '해설'), (Inches(1), Inches(0.3), Inches(5.7))),
            '기출문제세트': (3, ('번호', '정답', '해설'), (Inches(0.3), Inches(0.3), Inches(6.7)))            
        }



    def process_sections_dict_해설지(self, doc, sections_dict, last_section_title):
        #print("process_sections_dict_해설지 called")
        
        """
        Processes all sections in the sections_dict by adding them to the Word document.
        Handles the final section differently if needed.
        
        Args:
            doc (Document): The Word document being generated.
            sections_dict (dict): A dictionary mapping section titles to lists of (title, answer_part, explanation_part) tuples.
            last_section_title (str): The title of the last section for special handling.
        """
        try:
            section_titles = list(sections_dict.keys())  # Get all the section titles
            #self.app_logger.info("Section Titles: %s", section_titles)
        except IndexError:
            # Handle the error or pass if there's nothing specific to do
            messagebox.showerror("오류", "●추출할 지문이 없습니다. '추출할 지문번호 입력' 창에 지문번호가 올바르게 입력되어 있는지 확인하세요.")
            self.app_logger.info("●[오류] 추출할 지문이 없습니다.")
            return
        except Exception as e:
            # It's good practice to catch other potential errors
            self.app_logger.info(f"An unexpected error occurred: {e}")
            return

        try:
            # Filter out unwanted section titles
            filtered_section_titles = [
                title for title in section_titles 
                if "동반의어_Normal" not in title and 
                "영영정의_Normal" not in title and 
                "커스텀" not in title and 
                "요약문" not in title and title != "동반의어" and title != "영영정의"
            ]
            if filtered_section_titles:
                last_section_title = filtered_section_titles[-1]  # Get the last section title
            else:
                last_section_title = section_titles[-1]  # Fallback to the last section title
        except IndexError:
            # Handle the error or notify the user
            def nothing_to_export():
                error_dialog = Toplevel()
                error_dialog.title("오류")
                error_dialog.geometry("500x150")
                error_dialog.grab_set()  # Make the dialog modal
                error_dialog.focus_set()  # Make the dialog modal
                error_dialog.grid_rowconfigure(0, weight=1)
                error_dialog.grid_columnconfigure(0, weight=1)

                CTkLabel(
                    error_dialog, 
                    fg_color="white", 
                    width=300, 
                    height=120, 
                    text="-추출할 지문이 없습니다.\n'추출할 지문번호 입력'창에 추출할 지문번호가 올바르게 입력되어 있는지 확인하세요.", 
                    text_color="black"
                ).grid(row=0, column=0, sticky="nsew")

                # Frame for buttons
                button_frame = CTkFrame(error_dialog, fg_color="white", width=300, height=30, corner_radius=0)
                button_frame.grid(row=1, column=0, sticky="swe")
                button_frame.grid_columnconfigure(0, weight=1)

                # "OK" button action
                def on_ok():
                    self.close_export_popup = True
                    error_dialog.grab_release()
                    error_dialog.destroy()

                # Create "OK" button
                ok_button = CTkButton(
                    button_frame, 
                    border_width=2, 
                    border_color="gray", 
                    text="OK", 
                    text_color="black", 
                    width=70, 
                    height=30, 
                    command=on_ok, 
                    fg_color="#FEF9E0", 
                    hover_color="#DDA15C"
                )
                ok_button.grid(row=0, column=0, padx=10, pady=10)

            nothing_to_export()
            return
        except Exception as e:
            # It's good practice to catch other potential errors
            self.app_logger.info(f"An unexpected error occurred: {e}")
            return

        # Define column configurations if not already defined
        column_configurations = self.get_column_configurations()

        #print("sections_dict")
        #print(sections_dict)


        for section_title, title_answer_explanations in sections_dict.items():
            #print(f"section_title: {section_title}")    

            parts = section_title.split('. ', 1)
            if len(parts) > 1:
                base_section_title = parts[1]
            else:
                base_section_title = section_title  # Fallback if the format isn't as expected
            


            unwanted_sections = ["동반의어_Normal", "영영정의_Normal", "요약문", "커스텀"]

            if any(unwanted_section in section_title for unwanted_section in unwanted_sections):
                #해설지 스킵
                continue
            elif base_section_title == "동반의어" or base_section_title == "영영정의":
                continue

            
            if "실전모의고사" in section_title:
                self.handle_mock_test_section_해설지(doc, section_title, title_answer_explanations, column_configurations)
            elif "기출문제세트" in section_title:
                self.handle_past_test_section_해설지(doc, section_title, title_answer_explanations, column_configurations)
            else:
                self.handle_regular_section_해설지(doc, section_title, title_answer_explanations, column_configurations, sections_dict)

            self.add_page_break(doc)
            #if section_title != last_section_title:
                #print("Adding page break...")
                #self.add_page_break(doc)




    def 빠른정답찾기용(self, doc, sections_dict, last_section_title):

        try:
            section_titles = list(sections_dict.keys())  # Get all the section titles
            #self.app_logger.info("Section Titles: %s", section_titles)
        except IndexError:
            # Handle the error or pass if there's nothing specific to do
            messagebox.showerror("오류", "●추출할 지문이 없습니다. '추출할 지문번호 입력' 창에 지문번호가 올바르게 입력되어 있는지 확인하세요.")
            self.app_logger.info("●[오류] 추출할 지문이 없습니다.")
            return
        except Exception as e:
            # It's good practice to catch other potential errors
            self.app_logger.info(f"An unexpected error occurred: {e}")
            return

        try:
            # Filter out unwanted section titles
            filtered_section_titles = [
                title for title in section_titles 
                if "동반의어_Normal" not in title and 
                "영영정의_Normal" not in title and 
                "커스텀" not in title and 
                "요약문" not in title and title != "동반의어" and title != "영영정의"
            ]
            if filtered_section_titles:
                last_section_title = filtered_section_titles[-1]  # Get the last section title
            else:
                last_section_title = section_titles[-1]  # Fallback to the last section title
        except IndexError:
            # Handle the error or notify the user
            def nothing_to_export():
                error_dialog = Toplevel()
                error_dialog.title("오류")
                error_dialog.geometry("500x150")
                error_dialog.grab_set()  # Make the dialog modal
                error_dialog.focus_set()  # Make the dialog modal
                error_dialog.grid_rowconfigure(0, weight=1)
                error_dialog.grid_columnconfigure(0, weight=1)

                CTkLabel(
                    error_dialog, 
                    fg_color="white", 
                    width=300, 
                    height=120, 
                    text="-추출할 지문이 없습니다.\n'추출할 지문번호 입력'창에 추출할 지문번호가 올바르게 입력되어 있는지 확인하세요.", 
                    text_color="black"
                ).grid(row=0, column=0, sticky="nsew")

                # Frame for buttons
                button_frame = CTkFrame(error_dialog, fg_color="white", width=300, height=30, corner_radius=0)
                button_frame.grid(row=1, column=0, sticky="swe")
                button_frame.grid_columnconfigure(0, weight=1)

                # "OK" button action
                def on_ok():
                    self.close_export_popup = True
                    error_dialog.grab_release()
                    error_dialog.destroy()

                # Create "OK" button
                ok_button = CTkButton(
                    button_frame, 
                    border_width=2, 
                    border_color="gray", 
                    text="OK", 
                    text_color="black", 
                    width=70, 
                    height=30, 
                    command=on_ok, 
                    fg_color="#FEF9E0", 
                    hover_color="#DDA15C"
                )
                ok_button.grid(row=0, column=0, padx=10, pady=10)

            nothing_to_export()
            return
        except Exception as e:
            # It's good practice to catch other potential errors
            self.app_logger.info(f"An unexpected error occurred: {e}")
            return




        for section_title, title_answer_explanations in sections_dict.items():

            parts = section_title.split('. ', 1)
            if len(parts) > 1:
                base_section_title = parts[1]
            else:
                base_section_title = section_title  # Fallback if the format isn't as expected
            


            unwanted_sections = ["동반의어_Normal", "영영정의_Normal", "요약문", "커스텀"]

            if any(unwanted_section in section_title for unwanted_section in unwanted_sections):
                #해설지 스킵
                continue
            elif base_section_title == "동반의어" or base_section_title == "영영정의":
                continue


            self.빠른정답찾기(doc, section_title, title_answer_explanations)

            #if section_title != last_section_title:
                #print("Adding page break...")
                #self.add_page_break(doc)






    def 빠른정답찾기(self, doc, section_title, title_answer_explanations):
        print(f"빠른정답찾기 section_title: {section_title}")


        
        # 1) Special case: "빈칸암기" => skip table creation
        if "빈칸암기" in section_title:
            print("Detected '빈칸암기' in section_title -- skipping table creation.")
            answer_section_title = f"{section_title}_정답"
            doc.add_paragraph(answer_section_title, style='Heading_ans')
            para_title = doc.add_paragraph(style='Normal_modified')
            para_title.add_run("'정답 및 해설' 참조")
            return

        # Otherwise, create a heading
        answer_section_title = f"{section_title}_정답"
        doc.add_paragraph(answer_section_title, style='Heading_ans')

        # Decide table columns (default: 10, center alignment)
        col_count = 10
        horizontal_alignment = WD_ALIGN_PARAGRAPH.CENTER
        vertical_alignment = WD_ALIGN_VERTICAL.TOP
        use_explanation_for_second_row = False

        # "영작" => 5 columns, left-align, second row uses explanation_part
        if "영작" in section_title:
            #print("Detected '영작' in section_title -> 5 columns, left-align, second row = explanation_part.")
            col_count = 5
            horizontal_alignment = WD_ALIGN_PARAGRAPH.LEFT
            use_explanation_for_second_row = True

        # "영영정의문제" => 5 columns, left-align, second row = answer_part (default)
        elif "영영정의문제" in section_title:
            #print("Detected '영영정의문제' in section_title -> 5 columns, left-align.")
            col_count = 5
            horizontal_alignment = WD_ALIGN_PARAGRAPH.LEFT

        # ----------------------------
        # LINE-BY-LINE MODIFICATION
        # ----------------------------
        def modify_answer_part_line_by_line(ans: str, symbols):
            """
            Splits `ans` into lines, for each line:
            - find the first occurrence of any symbol in `symbols`
            - keep only the substring up to (and including) that symbol
            Then join the lines back.
            """
            lines = ans.splitlines()
            new_lines = []
            for line in lines:
                # Debug: show original line
                #print(f"  [modify_line] original: '{line}'")
                truncated_line = line
                found_index = None

                # For each symbol (e.g. ⓐ, ⓑ), find its index if it exists
                for sym in symbols:
                    idx = line.find(sym)
                    if idx != -1:
                        # If we find a symbol, note the earliest index
                        # (We only keep the earliest occurrence among all symbols)
                        if found_index is None or idx < found_index:
                            found_index = idx
                if found_index is not None:
                    # Keep everything up to and including the symbol
                    symbol = line[found_index : found_index+1]  # e.g., "ⓑ"
                    # But we also need the full length of that symbol (some are 1 char, some are 2, e.g. ③ might be 1 char in Unicode).
                    # If each symbol is 1 Unicode character, we can do:
                    truncated_line = line[: found_index + 1 + 0]  
                    # However, if the symbol is "③" (which is also 1 codepoint), then the length is 1.
                    # For safety, if your symbols are definitely 1 codepoint each, the above is okay.

                    #print(f"    Found symbol at index {found_index}, new_line = '{truncated_line}'")
                else:
                    print("    No symbol found in this line; keeping entire line.")
                new_lines.append(truncated_line)
            # Reconstruct multi-line string
            return "\n".join(new_lines)

        # For "어휘1단계", "어휘2단계", "어법1단계" => keep up to ⓐ or ⓑ
        def modify_answer_part_voca_grammar(ans: str) -> str:
            #print(f"[modify_answer_part_voca_grammar] multiline input:\n{ans}")
            return modify_answer_part_line_by_line(ans, ["ⓐ", "ⓑ"])

        # For "연결어" => keep up to ①, ②, ③, ④, or ⑤
        def modify_answer_part_connectives(ans: str) -> str:
            #print(f"[modify_answer_part_connectives] multiline input:\n{ans}")
            return modify_answer_part_line_by_line(ans, ["①","②","③","④","⑤"])

        # Combined approach
        def combined_modify_answer_part(ans: str) -> str:
            return ans  # default pass-through

        # If 어휘1단계, 어휘2단계, 어법1단계 => apply the voca/grammar logic
        if any(x in section_title for x in ["어휘1단계", "어휘2단계", "어법1단계"]):
            #print("Detected one of 어휘1단계 / 어휘2단계 / 어법1단계 in section_title.")
            old_func = combined_modify_answer_part
            def combined_modify_answer_part(ans: str) -> str:
                ans = old_func(ans)
                ans = modify_answer_part_voca_grammar(ans)
                return ans

        # If 연결어 => apply connectives logic
        if "연결어" in section_title:
            #print("Detected '연결어' in section_title.")
            old_func = combined_modify_answer_part
            def combined_modify_answer_part(ans: str) -> str:
                ans = old_func(ans)
                ans = modify_answer_part_connectives(ans)
                return ans

        # Finally, use `combined_modify_answer_part` in the table
        modify_answer_part = combined_modify_answer_part

        # Column widths
        widths10 = [Inches(0.7)] * 10
        widths5 = [Inches(1.4)] * 5

        chunk_size = col_count  # either 5 or 10
        total_items = len(title_answer_explanations)

        # Helper to shade rows
        def set_cell_shading(cell, fill_color):
            tc_pr = cell._tc.get_or_add_tcPr()
            shd = OxmlElement('w:shd')
            shd.set(qn('w:val'), 'clear')
            shd.set(qn('w:color'), "auto")
            shd.set(qn('w:fill'), fill_color)
            tc_pr.append(shd)

        # Build tables in chunks
        for i in range(0, total_items, chunk_size):
            chunk = title_answer_explanations[i : i + chunk_size]

            table = doc.add_table(rows=2, cols=col_count)

            # Row 0 => titles
            for col_idx, (title, answer_part, explanation_part) in enumerate(chunk):
                table.cell(0, col_idx).text = str(title)

            # Row 1 => answer_part (or explanation_part for "영작")
            for col_idx, (title, answer_part, explanation_part) in enumerate(chunk):
                if use_explanation_for_second_row:
                    text_line = str(explanation_part)
                else:
                    text_line = modify_answer_part(answer_part)

                table.cell(1, col_idx).text = text_line

            # Style, alignment, shading
            for row_idx in range(2):
                for col_idx in range(col_count):
                    cell = table.cell(row_idx, col_idx)
                    self.set_cell_paragraph_style(cell, 'Normal_exp')
                    self.set_cell_border(
                        cell,
                        top={"sz": 8, "val": "single", "color": "auto"},
                        bottom={"sz": 8, "val": "single", "color": "auto"},
                        left={"sz": 8, "val": "single", "color": "auto"},
                        right={"sz": 8, "val": "single", "color": "auto"}
                    )
                    self.set_cell_margins(cell, top=5, bottom=5, left=5, right=5)

                    cell.vertical_alignment = vertical_alignment
                    for paragraph in cell.paragraphs:
                        paragraph.alignment = horizontal_alignment

                    # shade row 0 => gray + bold
                    if row_idx == 0:
                        set_cell_shading(cell, "D9D9D9")
                        for paragraph in cell.paragraphs:
                            for run in paragraph.runs:
                                run.bold = True

            # Set column widths
            if col_count == 10:
                for row in table.rows:
                    for col_idx, width in enumerate(widths10):
                        row.cells[col_idx].width = width
            else:
                for row in table.rows:
                    for col_idx, width in enumerate(widths5):
                        row.cells[col_idx].width = width

            # Optional page break
            # self.add_page_break(doc)
            


    def show_error_and_log(self, message):
        messagebox.showerror("오류", message)
        self.app_logger.error(message)


    def process_text_replacement(self, file_path):
        print("process_text_replacement called")
        # Replace text in the document
        if self.text_school_grade or self.text_exam_type or self.text_week:
            doc = Document(file_path)
            for paragraph in doc.paragraphs:
                if paragraph.style.name == "School" and "2024 학교이름 ㅇ학년 ㅇ학기" in paragraph.text and self.text_school_grade:
                    paragraph.text = paragraph.text.replace("2024 학교이름 ㅇ학년 ㅇ학기", self.text_school_grade)
                elif paragraph.style.name == "Examtype" and "ㅇㅇ고사 내신대비" in paragraph.text and self.text_exam_type:
                    paragraph.text = paragraph.text.replace("ㅇㅇ고사 내신대비", self.text_exam_type)
                elif paragraph.style.name == "Week" and "ㅇ주차 숙제" in paragraph.text and self.text_week:
                    paragraph.text = paragraph.text.replace("ㅇ주차 숙제", self.text_week)
            doc.save(file_path)






    def merge_docs(self, content_path):
        #print("merge_docs called")

        ###### 표지 수정


        # 1. Make a temporary copy of the COVER_PATH file
        temp_cover_path = os.path.join(tempfile.gettempdir(), "temp_cover.docx")
#        print("Temporary cover path:", temp_cover_path)
        
        shutil.copy2(self.book_cover_path, temp_cover_path)

        # 2. Process text replacement on the temporary cover file
        try:
            self.process_text_replacement(temp_cover_path)
        except Exception as e:
            print(f"Error during text replacement: {e}")
            return


        # 3. Merge the two .docx files together
        cover_doc = Document(temp_cover_path)
        content_doc = Document(content_path)

        # Set document margins
        section = cover_doc.sections[0]
        section.top_margin = Cm(1.5)
        section.bottom_margin = Cm(1.6)
        section.left_margin = Cm(1.9)
        section.right_margin = Cm(1.9)

        # Define styles for cover_doc
        self.define_styles(cover_doc)

        try:
            blank_style = cover_doc.styles['blank']
            print("blank 스타일 발견")
        except KeyError:
            blank_style = cover_doc.styles.add_style('blank', WD_STYLE_TYPE.PARAGRAPH)
            print("blank 스타일 미발견")
        
        blank_style.font.name = "굴림"
        blank_style._element.rPr.rFonts.set(qn('w:eastAsia'), "굴림")
        blank_style.font.size = Pt(6)
        blank_style.paragraph_format.line_spacing_rule = WD_LINE_SPACING.MULTIPLE
        blank_style.paragraph_format.line_spacing = 1.0
        #if "hard" in str(self.book_cover_path):
        #    blank_style.paragraph_format.line_spacing = 1.15
        #else:
        #    blank_style.paragraph_format.line_spacing = 1.3
        
        blank_style.paragraph_format.space_before = Pt(0)
        blank_style.paragraph_format.space_after = Pt(0)
        blank_style.paragraph_format.alignment = WD_PARAGRAPH_ALIGNMENT.JUSTIFY

        # Append content_doc elements to cover_doc
        for element in content_doc.element.body:
            if not element.tag.endswith('sectPr'):
                cover_doc.element.body.append(element)

        # Apply styles to paragraphs
        for para in cover_doc.paragraphs:
            if self.is_answer_heading(para.text):
                para.style = cover_doc.styles["Heading_ans"]
            elif self.is_section_title(para.text) or self.is_answer_and_explanation(para.text)or self.fast_answer(para.text):
                para.style = cover_doc.styles["Heading 1"]

        # Remove blank lines before section titles
        for i, para in enumerate(cover_doc.paragraphs):
            if i > 0 and self.is_section_title(para.text) and not cover_doc.paragraphs[i - 1].text.strip():
                p = cover_doc.paragraphs[i - 1]._element
                p.getparent().remove(p)

        # Save the merged document
        cover_doc.save(content_path)

        # Delete the temporary cover file
        os.remove(temp_cover_path)
